/*
 * Copyright 2014 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.db.rmi;

import java.rmi.RemoteException;
import java.util.Date;

import de.willuhn.datasource.rmi.DBObject;

/**
 * Interface of the business object for history. According to the SQL table, we
 * define some getter an setter here.
 * 
 * <pre>
 * CREATE TABLE history (
 * id NUMERIC default UNIQUEKEY('history'),
 * aktien_id int(4) NOT NULL,
 * tradingDate DATE,
 * tradePrice double,
 * UNIQUE (id),
 * PRIMARY KEY (id)
 * );
 * </pre>
 * 
 * <br>
 * Getters and setters for the primary key (id) are not needed. Every one of the
 * following methods has to throw a RemoteException. <br>
 */
public interface ValueHistory extends DBObject {

	/**
	 * Liefert die Wertpapier.
	 * 
	 * @return die Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	Wertpapier getAktie() throws RemoteException;

	/**
	 * Liefert das TradingDate des ValueHistory.
	 * 
	 * @return tradingDate des ValueHistory.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	Date getTradingDate() throws RemoteException;

	/**
	 * Liefert den TradingPrice des ValueHistory.
	 * 
	 * @return tradingPrice des ValueHistory.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	double getTradingPrice() throws RemoteException;

	/**
	 * Setzt die Wertpapier
	 * 
	 * @param wertpapier
	 *            die Wertpapier
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setAktie(Wertpapier wertpapier) throws RemoteException;

	/**
	 * Setzt das tradindDate des ValueHistory.
	 * 
	 * @param tradingDate
	 *            des ValueHistory.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setTradingDate(Date tradingDate) throws RemoteException;

	/**
	 * Setzt den TradingPrice des ValueHistory.
	 * 
	 * @param tradingPrice
	 *            des ValueHistory.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setTradingPrice(double tradingPrice) throws RemoteException;

}
