/*
 * Copyright 2014 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.db.rmi;

import java.rmi.RemoteException;

import de.willuhn.datasource.rmi.DBObject;

/**
 * Interface fuer einen Datensatz in der Versionstabelle.
 * 
 * <pre>
 * CREATE TABLE version (
 *  id NUMERIC default UNIQUEKEY('version'),
 *  name varchar(255) NOT NULL,
 *  version int(5) NOT NULL,
 *  UNIQUE (id),
 *  PRIMARY KEY (id)
 * );
 * </pre>
  * <br>
 * Getters und Setters f&uuml;r den Primary Key (id) sind nicht n&ouml;tig. Jede Methode muss
 * zwingend eine RemoteException werfen.
 * <br>
 * * 
 */
public interface Version extends DBObject {

	/**
	 * Liefert den Namen der Version.
	 * 
	 * @return Name der Version.
	 * @throws RemoteException
	 */
	String getName() throws RemoteException;

	/**
	 * Speichert den Namen der Version.
	 * 
	 * @param name
	 *            Name der Version.
	 * @throws RemoteException
	 */
	void setName(String name) throws RemoteException;

	/**
	 * Liefert den aktuellen Stand der Version.
	 * 
	 * @return Stand der Version.
	 * @throws RemoteException
	 */
	 int getVersion() throws RemoteException;

	/**
	 * Legt die neue Versionsnummer fest.
	 * 
	 * @param newVersion
	 *            die neue Versionsnummer.
	 * @throws RemoteException
	 */
	 void setVersion(int newVersion) throws RemoteException;

}

