/*
 * Copyright 2014 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.db.server;

import java.io.File;
import java.rmi.RemoteException;
import java.sql.Connection;

import de.frankmuenster.jameica.depot.JaDepot;
import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.JaDepotDBService;
import de.willuhn.datasource.db.EmbeddedDBServiceImpl;
import de.willuhn.datasource.db.EmbeddedDatabase;
import de.willuhn.jameica.messaging.QueryMessage;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.plugin.PluginResources;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.sql.version.UpdateProvider;
import de.willuhn.sql.version.Updater;
import de.willuhn.util.ApplicationException;

/**
 * @author munstfr
 * 
 */
public class JaDepotDBServiceImpl extends EmbeddedDBServiceImpl implements
		JaDepotDBService {

	/**
	 * Serial UID
	 */
	private static final long serialVersionUID = 2458711982794157288L;

	/**
	 * @throws RemoteException
	 *             rmi-exception
	 */
	public JaDepotDBServiceImpl() throws RemoteException {
		super(Application.getPluginLoader().getPlugin(JaDepot.class)
				.getResources().getWorkPath()
				+ "/db/db.conf", "aktien", "aktview"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

		// We have to define jameicas classfinder.
		// otherwise, the db service will not be able to find
		// implementors by their interfaces.
		this.setClassFinder(Application.getClassLoader().getClassFinder());
	}

	/**
	 * @see JaDepotDBService#install()
	 */
	@Override
	public void install() throws RemoteException {

		// Let's create an embedded Database
		final PluginResources res = Application.getPluginLoader()
				.getPlugin(JaDepot.class).getResources();
		final Manifest manifest = Application.getPluginLoader().getManifest(
				JaDepot.class);
		EmbeddedDatabase database;

		try {
			database = new EmbeddedDatabase(res.getWorkPath() + "/db", //$NON-NLS-1$
					"aktien", "aktview"); //$NON-NLS-1$ //$NON-NLS-2$

			// create the sql tables.
			database.executeSQLScript(new File(manifest.getPluginDir()
					+ File.separator + "sql", "create.sql")); //$NON-NLS-1$ //$NON-NLS-2$

		} catch (Exception e) { // NOPMD
			throw new RemoteException(e.getMessage(), e.getCause()); // NOPMD
		}

	}

	/**
	 * @see JaDepotDBService#checkConsistency()
	 */
	@Override
	public void checkConsistency() throws RemoteException, ApplicationException {
		Logger.info(Settings.getI18n().tr(
				"JaDepotDBServiceImpl_startUpdateProviderInfo")); //$NON-NLS-1$
		final UpdateProvider provider = new JaDepotUpdateProvider(
				getConnection(), VersionUtil.getVersion(this, "db")); //$NON-NLS-1$
		final Updater updater = new Updater(provider, "iso-8859-1"); //$NON-NLS-1$
		updater.execute();
		Logger.info(Settings.getI18n().tr(
				"JaDepotDBServiceImpl_finishUpdateProviderInfo")); //$NON-NLS-1$

	}

	/**
	 * @see de.willuhn.datasource.db.DBServiceImpl#getConnection()
	 */
	@Override
	protected Connection getConnection() throws RemoteException {
		try {
			return super.getConnection();
		} catch (RemoteException re) {
			// Wir benachrichtigen Jameica ueber den Fehler, damit beim Shutdown
			// kein Backup erstellt wird
			Application.getMessagingFactory()
					.getMessagingQueue("jameica.error") //$NON-NLS-1$
					.sendMessage(new QueryMessage(re));
			throw re;
		}
	}
}
