package de.frankmuenster.jameica.depot.finanzen.currencyexchangerate;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * &Uuml;bersetzer Klasse zum ermitteln der W&auml;hrungenin der jeweilegen
 * Localen.
 * 
 * @author munstfr
 * @version 1.0
 */
public final class CurrencyName {

	/**
	 * Resource Pfad zu den W&auml;hrungsnamen
	 */
	private static final String RESOURCE_PATH = "lang.currency"; //$NON-NLS-1$
	//private static final String RESOURCE_PATH = "de/frankmuenster/jameica/depot/finanzen/currencyexchangerate/currency"; //$NON-NLS-1$

	/**
	 * Das Resource Bundle mit der Sprachdatei
	 */
	private static ResourceBundle bundle;

	static {
		try {
			bundle = ResourceBundle.getBundle(RESOURCE_PATH,
					Locale.getDefault());
		} catch (MissingResourceException e) {
			// Wir laden die deutsche Sprachdatei
			final Locale locale = new Locale("de", "DE"); //$NON-NLS-1$ //$NON-NLS-2$
			bundle = ResourceBundle.getBundle(RESOURCE_PATH, locale);

		}
	}

	private CurrencyName() {
		// Absichlich leer
	}

	/**
	 * Liefert den W&auml;hrungsnamen in der aktuellen Local
	 * 
	 * @param key
	 *            das W&aumlhrungsk&uuml;tzel
	 * @return den W&auml;hrungsnamen in der aktuellen Local
	 */
	public static String getString(final String key) {
		String result;
		try {
			result = bundle.getString(key);
		} catch (MissingResourceException e) {
			result = '!' + key + '!';
		}
		return result;
	}
}
