/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.controller;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierHistory;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceException;
import de.frankmuenster.jameica.depot.gui.input.PeriodChooserInput;
import de.frankmuenster.jameica.depot.gui.input.ZeitauswahlSchieberInput;
import de.frankmuenster.jameica.depot.gui.input.ZeitauswahlSchieberInputListener;
import de.frankmuenster.jameica.depot.gui.parts.AnalyseButtonPart;
import de.frankmuenster.jameica.depot.gui.parts.DiagrammAnzeigeAuswahlPart;
import de.frankmuenster.jameica.depot.gui.parts.KursHistoryGrafikPart;
import de.frankmuenster.jameica.depot.kursanalyse.Analysen;
import de.frankmuenster.jameica.depot.kursanalyse.IKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.MethodeAddRemoveListener;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.RadioInput;
import de.willuhn.logging.Logger;

/**
 * Kontroller f&uuml;r die Analysen-Ansicht.
 * 
 * @author munstfr
 * @version 1.0
 */
public class AnalysenKontroller extends AbstractControl implements
		MethodeAddRemoveListener {

	/**
	 * Das Wertpapier zur Analyse
	 */
	private transient Wertpapier wertpapier;

	/**
	 * Die Kurshistorien Grafik
	 */
	private transient KursHistoryGrafikPart grafik;

	/**
	 * Der Zeitauswahlschieber
	 */
	private transient ZeitauswahlSchieberInput zeitAuswahl;

	/**
	 * Diagramm-Anzeige Werkzeug
	 */
	private transient DiagrammAnzeigeAuswahlPart grafikWerkzeuge;

	/**
	 * Zeitauswahl-Kn6ouml;pfe
	 */
	private transient PeriodChooserInput zeitAuswahlBtns;

	/**
	 * Checkbox DAX anzeigen
	 */
	private transient CheckboxInput showDax;

	/**
	 * Checkbox Low-High anzeigen
	 */
	private transient CheckboxInput showLowHigh;

	/**
	 * Wertpapier History Werte
	 */
	private transient WertpapierHistory wertpapierHistory;

	/**
	 * Dax Histroy Werte
	 */
	private transient WertpapierHistory daxHistory;

	/**
	 * Analysemethoden Kn&ouml;pfe
	 */
	private transient AnalyseButtonPart analyseButtons;

	/**
	 * Der Wertpapier Name
	 */
	private final transient String aktienName;

	/**
	 * @param view
	 * @throws RemoteException
	 */
	public AnalysenKontroller(final AbstractView view) throws RemoteException {
		super(view);

		final String aktienSymbol = getAktie().getSymbol();
		aktienName = getAktie().getName();
		loadData(aktienSymbol);
		loadDaxData();

	}

	/**
	 * Return the AnalyseButtonPart to displaying the available AnalysButtons of
	 * this controller.
	 * 
	 * @return AnalyseButtonPart
	 */
	public AnalyseButtonPart getAnalyseButtons() {
		if (analyseButtons == null) {
			analyseButtons = new AnalyseButtonPart(this);
		}
		return analyseButtons;

	}

	/**
	 * Returns a KursHistoryGrafikPart of this controller.
	 * 
	 * @return das KursHistoryGrafikPart
	 */
	public KursHistoryGrafikPart getGrafik() {
		if (grafik == null) {

			grafik = new KursHistoryGrafikPart(aktienName, this);

			grafik.setKursDaxDaten(wertpapierHistory, daxHistory);
			grafik.setShowDax(false);
			grafik.setShowAsCandleStick(false);

			if (zeitAuswahl == null) {
				grafik.setRange(wertpapierHistory.getFromDate(),
						wertpapierHistory.getToDate());
			} else {
				grafik.setRange(zeitAuswahl.getLower(), zeitAuswahl.getUpper());
			}
		}
		return grafik;
	}

	/**
	 * Returns the ZeitauswahlSchieberInput of this controller.
	 * 
	 * @return das ZeitauswahlSchieberInput
	 */
	public ZeitauswahlSchieberInput getZeitauswahl() {
		if (zeitAuswahl == null) {

			zeitAuswahl = new ZeitauswahlSchieberInput();
			// Zeitachse setzten und das letzte Jahr auswhlen
			zeitAuswahl.setScaling(this.wertpapierHistory.getFromDate(),
					this.wertpapierHistory.getToDate(), 1, 10);
			zeitAuswahl.selectLastYear();
			// Platz fr den Kommentar schaffen
			zeitAuswahl.setComment(""); //$NON-NLS-1$
			zeitAuswahl.addListener(new ZeitAuswahlListener());
		}
		return zeitAuswahl;

	}

	/**
	 * Returns the DiagrammAnzeigeAuswahlPart of this controller
	 * 
	 * @return das DiagrammAnzeigeAuswahlPart
	 */
	public DiagrammAnzeigeAuswahlPart getGrafikWerkzeuge() {

		if (grafikWerkzeuge == null) {
			grafikWerkzeuge = new DiagrammAnzeigeAuswahlPart();
			grafikWerkzeuge.setDax(getShowDax());
			grafikWerkzeuge
					.setPeriod((PeriodChooserInput) getZeitauswahlButtons());
			grafikWerkzeuge.setHighLow(getShowLowHigh());
		}
		return grafikWerkzeuge;

	}

	private Part getZeitauswahlButtons() {

		if (zeitAuswahlBtns == null) {

			zeitAuswahlBtns = new PeriodChooserInput();
			zeitAuswahlBtns.addListener(new ZeitAuswahlButtonListener());
		}
		return zeitAuswahlBtns;

	}

	private CheckboxInput getShowDax() {

		if (showDax == null) {
			showDax = new CheckboxInput(false);
			showDax.setName("DAX"); //$NON-NLS-1$
			showDax.setEnabled(true);

			showDax.addListener(new Listener() {

				/**
				 * @see org.eclipse.swt.widgets.Listener#handleEvent
				 */
				@Override
				public void handleEvent(final Event event) {
					Logger.debug("DAX:" + showDax.getValue()); //$NON-NLS-1$

					if (event.type == SWT.Selection) {
						if ((Boolean) showDax.getValue()) {
							showLowHigh.setValue(false);
						}
						grafik.setShowDax((Boolean) showDax.getValue());
						grafik.setShowAsCandleStick((Boolean) showLowHigh
								.getValue());
						grafik.refresh();
					}
				}
			});
		}
		return showDax;
	}

	private CheckboxInput getShowLowHigh() {

		if (showLowHigh == null) {

			showLowHigh = new CheckboxInput(false);
			showLowHigh.setName("LowHigh"); //$NON-NLS-1$

			showLowHigh.addListener(new Listener() {

				/**
				 * @see org.eclipse.swt.widgets.Listener#handleEvent
				 */
				@Override
				public void handleEvent(final Event event) {
					Logger.debug("LowHigh:" + showLowHigh.getValue()); //$NON-NLS-1$
					if (event.type == SWT.Selection) {
						if ((Boolean) showLowHigh.getValue()) {
							showDax.setValue(false);
						}
						grafik.setShowDax((Boolean) showDax.getValue());
						grafik.setShowAsCandleStick((Boolean) showLowHigh
								.getValue());
						grafik.refresh();
					}
				}
			});
		}
		return showLowHigh;
	}

	/**
	 * Gibt das aktuelle Wertpaier zur&uuml;ck.
	 * 
	 * @return das Wertpapier.
	 */
	private Wertpapier getAktie() {
		if (wertpapier == null) {
			wertpapier = (Wertpapier) getCurrentObject();
		}
		return wertpapier;
	}

	private void loadData(final String symbol) throws RemoteException {

		if (StringUtils.isEmpty(symbol)) {
			throw new RemoteException(Settings.getI18n().tr(
					"AnalysenKontroller.symbolIsNull")); //$NON-NLS-1$
		}
		try {
			wertpapierHistory = new WertpapierHistory(symbol);
		} catch (YahooFinanceException e) {
			throw (RemoteException) new RemoteException(Settings.getI18n().tr(
					"AnalysenKontroller.stockHistoryLoadError")) //$NON-NLS-1$
					.initCause(e);
		}
	}

	private void loadDaxData() {

		try {
			daxHistory = new WertpapierHistory("^GDAXI"); //$NON-NLS-1$
		} catch (YahooFinanceException e) {
			GUI.getStatusBar().setErrorText(
					Settings.getI18n().tr(
							"AnalysenKontroller.daxHistoryLoadError")); //$NON-NLS-1$
			getShowDax().disable();
		}
	}

	/**
	 * Private Listener Klasse, die auf ZeitAuswahlEvents h&ouml;rt.
	 * 
	 * @author munstfr
	 * @version 1.0
	 */
	private class ZeitAuswahlListener implements
			ZeitauswahlSchieberInputListener {

		/**
		 * Standartkonstruktor
		 */
		public ZeitAuswahlListener() {
			// absichtlich leer
		}

		/**
		 * @see ZeitAuswahlListener#rangeChanged()
		 */
		@Override
		public void rangeChanged() {
			Logger.warn(Settings.getI18n().tr(
					"AnalysenKontroller.timeRangeChanged")); //$NON-NLS-1$
			grafik.setRange(zeitAuswahl.getLower(), zeitAuswahl.getUpper());
			grafik.refresh();
		}

	}

	/**
	 * Private Listener Klasse, die auf ZeitAuswahlButtonEvents h&ouml;rt.
	 * 
	 * @author munstfr
	 * @version 1.0
	 */
	private class ZeitAuswahlButtonListener implements Listener {

		/**
		 * Standartkonstruktor
		 */
		public ZeitAuswahlButtonListener() {
			// absichtlich leer
		}

		/**
		 * @see ZeitAuswahlButtonListener#handleEvent(Event)
		 */
		@Override
		public void handleEvent(final Event event) {

			if (event.type != SWT.FocusIn && event.type != SWT.FocusOut
					&& event.widget instanceof org.eclipse.swt.widgets.Button) {

				final int time = (Integer) RadioInput
						.getValue(PeriodChooserInput.GROUP_ID);
				Logger.warn(Settings
						.getI18n()
						.tr("AnalysenKontroller.timeRangeChoosen", Integer.toString(time))); //$NON-NLS-1$
				final int toDate = zeitAuswahl.getMaximun();

				zeitAuswahl.setUpper(toDate);
				zeitAuswahl.setLower(toDate - time);

				grafik.setRange(zeitAuswahl.getLower(), zeitAuswahl.getUpper());
				grafik.refresh();
			}
		}
	}

	/**
	 * @see MethodeAddRemoveListener#notifyAddMethode(IKursAnalyse)
	 */
	@Override
	@SuppressWarnings("unchecked")
	public void notifyAddMethode(final IKursAnalyse<?, ?> methode) {
		Logger.debug(Settings.getI18n().tr(
				"AnalysenKontroller.mothodAdded", methode.getLegendTitel())); //$NON-NLS-1$

		methode.setWertpapierHistory(wertpapierHistory);
		Analysen.getInstance().add((IKursAnalyse<Object, Object>) methode);

		analyseButtons.refresh();
		grafik.refresh();

		view.getParent().layout();
	}

	/**
	 * @see MethodeAddRemoveListener#notifyRemoveMethode(IKursAnalyse)
	 */
	@Override
	public void notifyRemoveMethode(final IKursAnalyse<?, ?> methode) {

		Analysen.getInstance().remove(methode);

		analyseButtons.refresh();
		grafik.refresh();

		view.getParent().layout();
	}

	/**
	 * @see MethodeAddRemoveListener#notifyRemoveAll()
	 */
	@Override
	public void notifyRemoveAll() {

		Analysen.getInstance().clear();

		analyseButtons.refresh();
		grafik.refresh();

		view.getParent().layout();
	}

}
