/* Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.controller;

import java.rmi.RemoteException;
import java.util.Date;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.CurrencyConverterYahoo;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.HistoryValueBean;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.StockInfoBean;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierHistory;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierInfoBean;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceAPI;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceException;
import de.frankmuenster.jameica.depot.gui.input.CurrencyInput;
import de.frankmuenster.jameica.depot.gui.parts.SymbolSearchPart;
import de.frankmuenster.jameica.depot.gui.parts.SymbolSearchPart.SearchResultSelectListener;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.frankmuenster.jameica.depot.wertpapierorder.Order;
import de.frankmuenster.jameica.depot.wertpapierorder.OrderImpl;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.DateInput;
import de.willuhn.jameica.gui.input.DecimalInput;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Kontroller fuer eine Order in der Detailansicht.
 * 
 * @author munstfr
 * @version 1.0
 */
@SuppressWarnings({ "PMD.ExcessiveImports", "PMD.GodClass", "PMD.TooManyFields" })
public class OrderController extends AbstractControl implements
		SearchResultSelectListener {

	/**
	 * Das Suchfeldpart
	 */
	private transient SymbolSearchPart symbolSearchPart;

	/**
	 * Das Yahoo-Symbol des Wertpapieres
	 */
	private transient TextInput symbol;

	/**
	 * Der Wertpapiername
	 */
	private transient TextInput name;

	/**
	 * Der Kauf-/Verkaufpreis
	 */
	private transient DecimalInput price;

	/**
	 * Die W&auml;hrung des Kauf-/Verkaufpreis. 
	 */
	private transient CurrencyInput stockCur;

	/**
	 * Der Kauf-/Verkaufpreis in der Hauptw&auml;hrung.
	 */
	private transient DecimalInput priceInMainCur;

	/**
	 * Umsatzvolumen
	 */
	private transient DecimalInput volume;

	/**
	 * Die zus&auml;tzliche Kosten
	 */
	private transient DecimalInput additionalCosts;

	/**
	 * Die zus&auml;tzliche Kosten in der Hauptw&auml;hrung.
	 */
	private transient DecimalInput addCostsInMainCur;

	/**
	 * Die W&auml;hrung der zus&auml;tzliche Kosten
	 */
	private transient CurrencyInput addCostsMainCur;

	/**
	 * Kauf-/Verkaufdatum
	 */
	private transient DateInput orderDate;

	/**
	 * Kommentar
	 */
	private transient TextAreaInput comment;

	/**
	 * Das Order-Objekt
	 */
	final private transient Order order;

	/**
	 * Das zuge&ouml;rigige Wertpapier-Objekt
	 */
	private transient Wertpapier wertpapier;

	/**
	 * Die W&auml;hrung des Wertpapiers
	 */
	private transient String wPapierWaehrung;

	/**
	 * Ist es ein neues Wertpapier?
	 */
	private transient boolean neuesWertpapier;

	/**
	 * Ist es eine neue Order
	 */
	private transient boolean neueWPapierOrder;

	/**
	 * Der Speicher-Button
	 */
	private transient Button store;

	/**
	 * Der Abbrechen-Button
	 */
	private transient Button cancle;

	/**
	 * Die Wertpapier historischen Daten als Feld, damit sie nur einmal geladen
	 * werden mssen.
	 */
	private transient WertpapierHistory wHistory;

	/**
	 * @param view
	 */
	public OrderController(final AbstractView view) {
		super(view);

		order = new OrderImpl();
		final Object currrentObject = getCurrentObject();
		if (currrentObject instanceof OrderHistory) {
			Logger.debug(Settings.getI18n().tr(
					"OrderController.readStockOrderFromDbInfo")); //$NON-NLS-1$
			final OrderHistory orderHistory = (OrderHistory) getCurrentObject();
			try {
				wertpapier = orderHistory.getAktie();
				wPapierWaehrung = wertpapier.getCurrency();
				order.setAdditionalCosts(orderHistory.getAdditionalCosts());
				order.setAdditionalCostsInMainCurrency(orderHistory
						.getAdditionalCostsInMainCurrency());
				order.setAdditionalCostsCurrency(orderHistory
						.getAdditionalCostsCurrency());
				order.setName(orderHistory.getAktie().getName());
				order.setOrderDate(orderHistory.getOrderDate());
				order.setOrderPrice(orderHistory.getOrderPrice());
				order.setOrderPriceInMainCurrency(orderHistory
						.getOrderPriceInMainCurrency());
				order.setOrderVolume(orderHistory.getOrderAmount());
				order.setSymbol(orderHistory.getAktie().getSymbol());
				order.setComment(orderHistory.getComment());
			} catch (RemoteException e) {
				Logger.warn(Settings.getI18n().tr(
						"OrderController.readStockOrderFromDbWarning", //$NON-NLS-1$
						e.getMessage()));
			}
			neuesWertpapier = false;
			neueWPapierOrder = false;
		} else if (currrentObject instanceof Wertpapier) {
			Logger.debug(Settings.getI18n().tr(
					"OrderController.readStockFromDbCreateNewOrderInfo")); //$NON-NLS-1$
			wertpapier = (Wertpapier) currrentObject;
			try {
				wPapierWaehrung = wertpapier.getCurrency();
				order.setAdditionalCosts(0.0d);
				order.setAdditionalCostsCurrency(wPapierWaehrung);
				order.setSymbol(wertpapier.getSymbol());
				order.setName(wertpapier.getName());
				order.setOrderVolume(0.0d);

				// Whrung und aktuellen Kurs ermitteln
				try {
					final WertpapierInfoBean wPapierInfoBean = YahooFinanceAPI
							.getInstance().getWertpapierInfo(
									wertpapier.getSymbol(), false);
					if (wPapierInfoBean != null) {
						// neuesWertpapier.setExchange(wertpapierInfo.getStockExchange());
						order.setOrderDate(wPapierInfoBean.getLastTradeDate());
						order.setOrderPrice(wPapierInfoBean
								.getLastTradePriceOnly());
						handlePriceChanged();
					}
				} catch (YahooFinanceException ye) {
					// OK, dann halt nicht.
					order.setOrderDate(new Date());
					order.setOrderPrice(0.0d);
				}
			} catch (RemoteException e) {
				Logger.error(Settings.getI18n().tr(
						"OrderController.readStockFromDbCreateNewOrderError" //$NON-NLS-1$
						, e.getMessage()));
			}
			neuesWertpapier = false;
			neueWPapierOrder = true;
		} else {
			Logger.debug(Settings.getI18n().tr(
					"OrderController.newStockOrderInfo")); //$NON-NLS-1$
			neuesWertpapier = true;
			neueWPapierOrder = true;
			wPapierWaehrung = Settings.getDepotWaehrung();
			// wertpapier = null;
			// wPapierWaehrung = null;
		}
	}

	/**
	 * @return symbolSearchPart
	 */
	public SymbolSearchPart getSymbolSearchPart() {

		if (symbolSearchPart == null) {
			symbolSearchPart = new SymbolSearchPart(this);
		}
		return symbolSearchPart;
	}

	/**
	 * @return symbol
	 */
	public TextInput getSymbol() {
		if (symbol == null) {
			symbol = new TextInput(order.getSymbol());
			if (!isNeuesWertpapier()) {
				symbol.setEnabled(false);
			}
		}
		return symbol;
	}

	/**
	 * @return name
	 */
	public TextInput getName() {
		if (name == null) {
			name = new TextInput(order.getName());
			if (!isNeuesWertpapier()) {
				name.setEnabled(false);
			}
		}
		return name;
	}

	/**
	 * @return price
	 */
	public DecimalInput getPrice() {
		if (price == null) {
			price = new DecimalInput(order.getOrderPrice(), FormatterUtils.DECIMAL_FORMAT_6.get());
//			if (StringUtils.isNotEmpty(wPapierWaehrung)) {
//				price.setComment(wPapierWaehrung);
//			} else {
//				price.setComment(""); // Platz lassen fr die //$NON-NLS-1$
//										// Whrung
//			}
			price.addListener(new Listener() {
				/**
				 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event
				 *      event)
				 */
				@Override
				public void handleEvent(final Event event) {

					if (event.type == SWT.FocusIn) {
						price.hasChanged();
					} else if (event.type == SWT.FocusOut && price.hasChanged()) {
						handlePriceChanged();
					}
				}
			});
		}
		return price;
	}
	
	/**
	 * Whrung 
	 * @return Whrungsauswahlfeld
	 */
	public CurrencyInput getStockCurrency() {
		if (stockCur == null) {
			stockCur = new CurrencyInput(wPapierWaehrung);
			stockCur.addListener(new Listener() {

				/**
				 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event
				 *      event)
				 */
				@Override
				public void handleEvent(final Event event) {
					Logger.warn(event.toString());
					if (event.type == SWT.Selection) {
						wPapierWaehrung = (String) stockCur.getValue();
						handleCostsChanged();
					}
				}
			});
			
			if (!neuesWertpapier) {
				stockCur.setEnabled(false);
			}
		}
		return stockCur;
	}

	
	/**
	 * @return priceInMainCur
	 */
	public DecimalInput getPriceInMainCurrency() {
		if (priceInMainCur == null) {
			priceInMainCur = new DecimalInput(
					order.getOrderPriceInMainCurrency(), FormatterUtils.DECIMAL_FORMAT_6.get());
			priceInMainCur.setComment(""); //$NON-NLS-1$
		}
		return priceInMainCur;
	}

	/**
	 * @return volume
	 */
	public DecimalInput getVolume() {
		if (volume == null) {
			volume = new DecimalInput(order.getOrderVolume(),
					FormatterUtils.DECIMAL_FORMAT_6.get());
		}
		return volume;
	}

	/**
	 * @return additionalCosts
	 */
	public DecimalInput getAdditionalCosts() {
		if (additionalCosts == null) {
			additionalCosts = new DecimalInput(order.getAdditionalCosts(),
					FormatterUtils.DECIMAL_FORMAT_6.get());
			additionalCosts.addListener(new Listener() {

				/**
				 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event
				 *      event)
				 */
				@Override
				public void handleEvent(final Event event) {
					if (event.type == SWT.FocusIn) {
						additionalCosts.hasChanged();
					} else if (event.type == SWT.FocusOut
							&& additionalCosts.hasChanged()) {
						handleCostsChanged();
					}
				}
			});
		}
		return additionalCosts;
	}

	/**
	 * @return additionalCostsInMainCurrrency
	 */
	public DecimalInput getAdditionalCostsInMainCurrency() {
		if (addCostsInMainCur == null) {
			addCostsInMainCur = new DecimalInput(
					order.getAdditionalCostsInMainCurrency(),
					FormatterUtils.DECIMAL_FORMAT_6.get());
			addCostsInMainCur.setComment(StringUtils.EMPTY);
		}
		return addCostsInMainCur;
	}

	/**
	 * @return additionalCosts
	 */
	public CurrencyInput getAdditionalCostsCurrency() {
		if (addCostsMainCur == null) {
			addCostsMainCur = new CurrencyInput(
					order.getAdditionalCostsCurrency());
			addCostsMainCur.addListener(new Listener() {

				/**
				 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event
				 *      event)
				 */
				@Override
				public void handleEvent(final Event event) {
					Logger.warn(event.toString());
					if (event.type == SWT.Selection) {
						handleCostsChanged();
					}
				}
			});
		}
		return addCostsMainCur;
	}

	/**
	 * @return orderDate
	 */
	public DateInput getOrderDate() {
		if (orderDate == null) {
			orderDate = new DateInput(order.getOrderDate(), FormatterUtils.GERMAN_DATE_FORMAT.get());
			orderDate.addListener(new Listener() {

				/**
				 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event
				 *      event)
				 */
				@Override
				public void handleEvent(final Event event) {

					Logger.debug(event.toString());
					if (event.type == SWT.Selection) {
						handleDateChanged();
					}
				}
			});
		}
		return orderDate;
	}

	/**
	 * @return comment
	 */
	public TextAreaInput getComment() {
		if (comment == null) {
			comment = new TextAreaInput(order.getComment());
		}
		return comment;
	}

	/**
	 * Liefert den Speichern-Button
	 * 
	 * @return den Speichern-Button
	 */
	public Button getStore() {
		if (store == null) {
			store = new Button(
					Settings.getI18n().tr("OrderController.saveBtn"), new Action() { //$NON-NLS-1$

						/**
						 * @see Action#handleAction(Object context)
						 */
						@Override
						public void handleAction(final Object context)
								throws ApplicationException {
							handleStore();

						}
					}, null, true, "save-icon.png"); //$NON-NLS-1$
			store.setEnabled(true);
		}
		return store;
	}

	/**
	 * Liefert den Abbrechen-Button
	 * 
	 * @return den Abbrechen-Button
	 */
	public Button getCancel() {

		if (cancle == null) {

			cancle = new Button(Settings.getI18n().tr(
					Settings.getI18n().tr("OrderController.cancelBtn")), //$NON-NLS-1$
					new Action() {

						/**
						 * @see Action#handleAction(Object context)
						 */
						@Override
						public void handleAction(final Object context)
								throws ApplicationException {
							if (isNeuesWertpapier()) {
								GUI.startView(
										de.frankmuenster.jameica.depot.gui.views.DepotView.class,
										null);
							} else {
								GUI.startView(
										de.frankmuenster.jameica.depot.gui.views.WertpapierDetailsView.class,
										wertpapier);
							}
						}
					}, null, false, "cancle-icon.png"); //$NON-NLS-1$

		}
		return cancle;

	}

	/**
	 * Ist es ein neues Wertpapier
	 * 
	 * @return TRUE, falls es sich auch um ein neues Wertpapier handelt.
	 */
	public boolean isNeuesWertpapier() {
		return neuesWertpapier;
	}

	/**
	 * Ist es eine neue Order
	 * 
	 * @return TRUE, falls es sich auch um eine neue Ordner handelt.
	 */
	public boolean isNeueWertpapierOrder() {
		return neueWPapierOrder;
	}

	/**
	 * Aktualisiert die Anzeige
	 */
	public void updateView() {
		handleDateChanged();
		handlePriceChanged();
		handleCostsChanged();
	}

	private void handleStore() {

		// interne order Instance aktualisieren
		order.setAdditionalCosts((Double) getAdditionalCosts().getValue());
		order.setName((String) getName().getValue());
		order.setOrderDate((Date) getOrderDate().getValue());
		order.setOrderPrice((Double) getPrice().getValue());
		order.setOrderVolume((Double) getVolume().getValue());
		order.setSymbol((String) getSymbol().getValue());
		order.setOrderPriceInMainCurrency((Double) getPriceInMainCurrency()
				.getValue());
		order.setAdditionalCostsCurrency((String) getAdditionalCostsCurrency()
				.getValue());
		order.setAdditionalCostsInMainCurrency((Double) getAdditionalCostsInMainCurrency()
				.getValue());

		try {

			final DBService service = Settings.getDBService();
			final Wertpapier wertpapier = getAktie(service, order.getSymbol());

			if (Logger.isLogging(Level.INFO)) {
				Logger.info(Settings.getI18n().tr(
						"OrderController.storeOrderinfo") + order.toString()); // NOPMD //$NON-NLS-1$
			}

			OrderHistory orderHistory;
			// ist neue Order?
			if (isNeuesWertpapier()) {
				orderHistory = (OrderHistory) service.createObject(
						OrderHistory.class, null);
			} else if (isNeueWertpapierOrder()) {
				order.setComment((String) getComment().getValue());
				orderHistory = (OrderHistory) service.createObject(
						OrderHistory.class, null);
			} else {
				order.setComment((String) getComment().getValue());
				orderHistory = (OrderHistory) getCurrentObject();
			}

			orderHistory.setAktie(wertpapier);
			orderHistory.setOrderAmount(order.getOrderVolume());
			orderHistory.setOrderDate(order.getOrderDate());
			orderHistory.setOrderPrice(order.getOrderPrice());
			orderHistory.setOrderPriceInMainCurrency(order
					.getOrderPriceInMainCurrency());
			orderHistory.setAdditionalCosts(order.getAdditionalCosts());
			orderHistory.setAdditionalCostsCurrency(order
					.getAdditionalCostsCurrency());
			orderHistory.setAdditionalCostsInMainCurrency(order
					.getAdditionalCostsInMainCurrency());
			orderHistory.setComment(order.getComment());
			try {
				orderHistory.store();
				Application.getMessagingFactory().sendMessage(
						new StatusBarMessage(Settings.getI18n().tr(
								"OrderController.storeOrderSuccess"), //$NON-NLS-1$
								StatusBarMessage.TYPE_SUCCESS));

			} catch (ApplicationException e) {
				Application.getMessagingFactory().sendMessage(
						new StatusBarMessage(e.getMessage(),
								StatusBarMessage.TYPE_ERROR));
			}

			// und zurck zur Wertpapier-Detailansicht
			GUI.startView(
					de.frankmuenster.jameica.depot.gui.views.WertpapierDetailsView.class,
					wertpapier);

		} catch (RemoteException e) {
			Logger.error(
					Settings.getI18n().tr("OrderController.storeStockError"), e); //$NON-NLS-1$
			Application
					.getMessagingFactory()
					.sendMessage(
							new StatusBarMessage(
									Settings.getI18n()
											.tr("OrderController.storeStockErrorMsg", e.getMessage()), //$NON-NLS-1$
									StatusBarMessage.TYPE_ERROR));
		} catch (Exception e) { // NOPMD
			Logger.error(
					Settings.getI18n().tr("OrderController.storeStockError"), e); //$NON-NLS-1$
			Application
					.getMessagingFactory()
					.sendMessage(
							new StatusBarMessage(
									Settings.getI18n()
											.tr("OrderController.storeStockErrorMsg", e.getMessage()), //$NON-NLS-1$
									StatusBarMessage.TYPE_ERROR));
		}

	}

	private Wertpapier getAktie(final DBService service, final String symbol)
			throws Exception { // NOPMD

		Wertpapier wertpapier = null;
		final DBIterator<?> aktien = service.createList(Wertpapier.class);

		while (aktien.hasNext()) {
			final Wertpapier wertpapierTest = (Wertpapier) aktien.next();
			Logger.warn(wertpapierTest.getSymbol() + " - " + symbol); //$NON-NLS-1$
			if (wertpapierTest.getSymbol().equals(symbol)) {
				wertpapier = wertpapierTest;
			}
		}

		if (wertpapier == null) {
			// Es muss ein neues WP angelegt werden.
			wertpapier = (Wertpapier) service.createObject(Wertpapier.class,
					null);
			wertpapier.setSymbol(order.getSymbol());
			wertpapier.setName(order.getName());
			wertpapier.setCurrency(wPapierWaehrung);
			wertpapier.setComment((String) getComment().getValue());

			// Aktuellen Kurs ermitteln
			try {
				final WertpapierInfoBean wPapierInfoBean = YahooFinanceAPI
						.getInstance().getWertpapierInfo(
								wertpapier.getSymbol(), false);
				if (wPapierInfoBean != null) {
					// neuesWertpapier.setExchange(wertpapierInfo.getStockExchange());
					wertpapier.setLastTradePriceOnly(wPapierInfoBean
							.getLastTradePriceOnly());
					wertpapier.setLastTradeDate(wPapierInfoBean
							.getLastTradeDate());
				}
			} catch (YahooFinanceException ye) {
				// OK, dann halt nicht.
				Logger.info(Settings.getI18n().tr(
						"OrderController.noFinancialInfoFound")); //$NON-NLS-1$
			}

			wertpapier.store();
		}

		return wertpapier;
	}

	/**
	 * @see SearchResultSelectListener#searchResultSelected(StockInfoBean)
	 */
	@Override
	public void searchResultSelected(final StockInfoBean stockInfoBean) {
		Logger.warn(Settings
				.getI18n()
				.tr("OrderController.searchResultSelectedInfo", stockInfoBean.toString())); //$NON-NLS-1$
		symbol.setValue(stockInfoBean.getSymbol());
		name.setValue(stockInfoBean.getName());

		// Whrung und aktuellen Kurs ermitteln
		try {
			final WertpapierInfoBean wPapierInfoBean = YahooFinanceAPI
					.getInstance().getWertpapierInfo(stockInfoBean.getSymbol(),
							false);
			if (wPapierInfoBean != null) {
				// neuesWertpapier.setExchange(wertpapierInfo.getStockExchange());
				wPapierWaehrung = wPapierInfoBean.getCurrency();
				stockCur.setValue(wPapierWaehrung);
				price.setValue(wPapierInfoBean.getLastTradePriceOnly());
				price.setComment(wPapierWaehrung);
				orderDate.setValue(wPapierInfoBean.getLastTradeDate());
				handlePriceChanged();
			}
		} catch (YahooFinanceException ye) {
			// Melden, das Kurs nicht gefunden wurde
			final StringBuffer msg = new StringBuffer();
			msg.append(Settings.getI18n().tr(
					"OrderController.stockInfoNotFoundMsg", (String) symbol.getValue())); //$NON-NLS-1$
			Application.getMessagingFactory().sendMessage(
					new StatusBarMessage(msg.toString(),
							StatusBarMessage.TYPE_INFO));
			price.setValue(0.0d);
			price.setComment(""); //$NON-NLS-1$
			orderDate.setValue(null);
			wPapierWaehrung = Settings.getDepotWaehrung();
			handlePriceChanged();
		}

	}

	private void handleDateChanged() {

		final String symbol = (String) getSymbol().getValue();
		if (StringUtils.isNotEmpty(symbol)) {

			Logger.debug(((Date) getOrderDate().getValue()).toString());

			// Versuch den Kurswert an dem Datum zu ermitteln
			try {
				if (wHistory == null || !(wHistory.getSymbol().equals(symbol))) {
					wHistory = new WertpapierHistory(symbol);
				}
				final Date setDate = (Date) getOrderDate().getValue();
				final HistoryValueBean value = wHistory
						.getValuesAtDate(setDate);
				if (value != null) {
					price.setValue(value.getClose());
					orderDate.setValue(value.getDate());

					// Informiere Anwender falls Werte nicht taggenau
					if (!DateUtils.isSameDay(setDate, value.getDate())) {
						final String valueDateString = FormatterUtils.GERMAN_DATE_FORMAT
								.get().format(value.getDate());
						final String msg = Settings.getI18n().tr(
								"OrderController.valueOnDateNotInHistory", //$NON-NLS-1$
								valueDateString);
						Application.getMessagingFactory().sendMessage(
								new StatusBarMessage(msg,
										StatusBarMessage.TYPE_INFO));
					}
					handlePriceChanged();
				}
			} catch (YahooFinanceException ye) {
				// Melden, das Kurs nicht gefunden wurde
				final StringBuffer msg = new StringBuffer();
				msg.append(Settings.getI18n().tr(
						"OrderController.stockInfoNotFoundMsg", symbol)); //$NON-NLS-1$
				Application.getMessagingFactory().sendMessage(
						new StatusBarMessage(msg.toString(),
								StatusBarMessage.TYPE_INFO));
			}
		}
	}

	private void handlePriceChanged() {

		if (priceInMainCur != null && priceInMainCur.getControl() != null) {
			if (wPapierWaehrung.equals(Settings.getDepotWaehrung())) {
				priceInMainCur.setComment(StringUtils.EMPTY);
				priceInMainCur.getControl().setVisible(false);
			} else {
				if (StringUtils.isNotEmpty(wPapierWaehrung)
						&& getOrderDate().getValue() != null) {

					//priceInMainCur.getControl().setVisible(true);
					final Date orderDate = (Date) getOrderDate().getValue();

					try {
						final float exRate = CurrencyConverterYahoo
								.getInstance().exchangeRate(wPapierWaehrung,
										Settings.getDepotWaehrung(), orderDate);

						final Double price = (Double) getPrice().getValue();
						final Double exchangedPrice = price * exRate;

						getPriceInMainCurrency().setValue(exchangedPrice);
						priceInMainCur.setComment(Settings.getI18n().tr(
								"OrderController.in") //$NON-NLS-1$
								+ Settings.getDepotWaehrung());
						priceInMainCur.getControl().setVisible(true);

					} catch (YahooFinanceException e) {
						if (Logger.isLogging(Level.INFO)) {
							Logger.info(Settings.getI18n().tr(
									"OrderController.exchangeRateNotFound", //$NON-NLS-1$
									e.getMessage()));
						}
					}
				} else {
					priceInMainCur.setComment(StringUtils.EMPTY);
					priceInMainCur.getControl().setVisible(false);
				}
			}
		}
	}

	private void handleCostsChanged() {

		if (((String) getAdditionalCostsCurrency().getValue()).equals(Settings
				.getDepotWaehrung())) {
			addCostsInMainCur.setComment(StringUtils.EMPTY); 
			addCostsInMainCur.getControl().setVisible(false);
			return;
		}

		if (getAdditionalCostsCurrency().getValue() == null
				|| getOrderDate().getValue() == null) {
			addCostsInMainCur.setComment(StringUtils.EMPTY); 
			addCostsInMainCur.getControl().setVisible(false);
		} else {

			final Date orderDate = (Date) getOrderDate().getValue();

			try {
				final float exRate = CurrencyConverterYahoo.getInstance()
						.exchangeRate(
								(String) getAdditionalCostsCurrency()
										.getValue(),
								Settings.getDepotWaehrung(), orderDate);

				final Double costs = (Double) getAdditionalCosts().getValue();

				addCostsInMainCur.setValue(costs * exRate);
				addCostsInMainCur.setComment(Settings.getI18n().tr(
						"OrderController.in") //$NON-NLS-1$
						+ Settings.getDepotWaehrung());
				addCostsInMainCur.getControl().setVisible(true);

			} catch (YahooFinanceException e) {
				Logger.error(Settings.getI18n().tr(
						"OrderController.exchangerateCalculationFailed", //$NON-NLS-1$
						e.getMessage()));
			}
		}
	}
}
