/*
 * Copyright 2016 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.dialog;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.JaDepot;
import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Version;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.LabelGroup;
import de.willuhn.jameica.gui.util.SWTUtil;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.system.Application;
import de.willuhn.util.ApplicationException;

/**
 * "AboutDialog..." Dialog. Basiert auf dem About dialog aus Jameica von Olaf
 * Willuhn
 * 
 * @author willuhn/munstfr
 * @version 1.1
 * 
 */
public class AboutDialog extends AbstractDialog<Object> {

	/**
	 * Liefetr einen modalen Dialog, der mittel .open() angezeigt werden kann
	 * 
	 * @param position
	 *            die Position, an der der Dialog angezeigt werden soll.
	 *            Mgliche Werte sind POSITION_MOUSE und POSITION_CENTER.
	 */
	public AboutDialog(final int position) {

		super(position);

		final AbstractPlugin plugin = Application.getPluginLoader().getPlugin(
				JaDepot.class);

		this.setTitle(Settings.getI18n().tr("AboutDialog.about")); //$NON-NLS-1$
		this.setPanelText(Settings.getI18n().tr("AboutDialog.panelText", //$NON-NLS-1$
				plugin.getManifest().getVersion().toString()));
	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
	 */
	@Override
	protected Object getData() {
		return null;
	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected void paint(final Composite parent) throws RemoteException {

		final AbstractPlugin plugin = Application.getPluginLoader().getPlugin(
				JaDepot.class);

		final DBIterator<?> list = Settings.getDBService().createList(
				Version.class);
		list.addFilter("name = ?", "db"); //$NON-NLS-1$ //$NON-NLS-2$
		final Version version = (Version) list.next();

		final Container image = new SimpleContainer(parent, false, 2);
		final GridLayout glImage = new GridLayout();
		glImage.numColumns = 2;
		image.getComposite().setLayout(glImage);
		final CLabel imageLabel = new CLabel(image.getComposite(), SWT.NONE);
		final GridData gdImage = new GridData();
		gdImage.grabExcessHorizontalSpace = true;
		gdImage.horizontalAlignment = SWT.RIGHT;
		gdImage.horizontalSpan = 2;

		imageLabel.setImage(SWTUtil.getImage("jadepot.png")); //$NON-NLS-1$
		imageLabel.setMargins(20, 10, 20, 20);

		// final Label label = GUI.getStyleFactory().createLabel(parent,
		// SWT.BORDER);
		// label.setImage(SWTUtil.getImage("jadepot.png"));

		final Container container = new LabelGroup(parent, Settings.getI18n()
				.tr(Settings.getI18n().tr("AboutDialog.versionInfo")), true); //$NON-NLS-1$

		final StringBuffer formString = new StringBuffer(300) // NOPMD 
				.append("<form><p><b>") //$NON-NLS-1$
				.append(Settings.getI18n().tr("AboutDialog.appTitel")) //$NON-NLS-1$
				.append("</b></p><p>") //$NON-NLS-1$
				.append(Settings.getI18n().tr("AboutDialog.lisence")) //$NON-NLS-1$

				.append(" [<a href=\"http://ec.europa.eu/idabc/eupl\">http://ec.europa.eu/idabc/eupl</a>]<br/>") //$NON-NLS-1$
				.append(Settings.getI18n().tr("AboutDialog.copyright")) //$NON-NLS-1$

				.append(" Frank Mnster [<a href=\"mailto:jameica@frankmuenster.de\">jameica@frankmuenster.de</a>]<br/>") //$NON-NLS-1$
				.append(Settings.getI18n().tr("AboutDialog.swVersion")) //$NON-NLS-1$
				.append(" ") //$NON-NLS-1$
				.append(plugin.getManifest().getVersion())
				.append("<br/>Build: ") //$NON-NLS-1$
				.append(plugin.getManifest().getBuildnumber()).append(" [") //$NON-NLS-1$
				.append(Settings.getI18n().tr("AboutDialog.date")) //$NON-NLS-1$
				.append(" ").append(plugin.getManifest().getBuildDate()) //$NON-NLS-1$
				.append("]<br/>") //$NON-NLS-1$
				.append(Settings.getI18n().tr("AboutDialog.dbVersion")) //$NON-NLS-1$
				.append(" ").append( //$NON-NLS-1$
						version.getVersion()).append("</p></form>"); //$NON-NLS-1$
		final FormTextPart text = new FormTextPart();
		text.setText(formString.toString());
		// "<form>"
		// + "<p><b>JaDepot - Plugin fr Jameica</b></p>"
		// +
		// "<p>Lizenz: EUPL [<a href=\"http://ec.europa.eu/idabc/eupl\">http://ec.europa.eu/idabc/eupl</a>]<br/>"
		// +
		// "Copyright by Frank Mnster [<a href=\"mailto:jameica@frankmuenster.de\">jameica@frankmuenster.de</a>]<br/>"
		// +
		// "<a href=\"http://progs.frankmuenster.de/jameica-jadepot/\">http://progs.frankmuenster.de/jameica-jadepotr</a></p>"
		// + "<p>Software-Version: " + plugin.getManifest().getVersion()
		// + "<br/>" + "Build: " + plugin.getManifest().getBuildnumber()
		// + " [Datum " + plugin.getManifest().getBuildDate() + "]"
		// + "<br/>Datenbank-Version: " + version.getVersion() + "</p>"
		// + "</form>");
		container.addPart(text);

		final ButtonArea buttons = new ButtonArea();
		buttons.addButton(
				Settings.getI18n().tr("AboutDialog.close"), new Action() { //$NON-NLS-1$
					/**
					 * @see de.willuhn.jameica.gui.Action#handleAction(Object
					 *      context)
					 */
					@Override
					public void handleAction(final Object context)
							throws ApplicationException {
						close();
					}
				}, null, true, "window-close.png"); //$NON-NLS-1$

		container.addButtonArea(buttons);

		setSize(SWT.DEFAULT, 520);
	}
}
