/*
 * Copyright 2016 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.input;

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.Currency;
import de.willuhn.jameica.gui.input.SelectInput;

/**
 * Auswahlfeld f&uuml;r die W&auml;hrung.
 */
public class CurrencyInput extends SelectInput {

	/**
	 * Konstruktor mit Angabe der W&auml;hrung
	 * 
	 * @param value
	 *            die vorausgew&auml;hlte W&auml;hrung.
	 */
	public CurrencyInput(final String value) {
		super(getCodes(), value);
		this.setName(Settings.getI18n().tr("CurrencyInput.currency")); //$NON-NLS-1$
		this.setEditable(false); // Man kann selbst keine Werte eingeben
		this.setComment(""); //$NON-NLS-1$

		final Listener listener = new Listener() {
			/**
			 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event event)
			 */
			@Override
			public void handleEvent(final Event event) {
				final String value = (String) getValue();
				final Currency currency = Currency.find(value);
				setComment(currency == null ? StringUtils.EMPTY : currency.getName()); //$NON-NLS-1$
			}
		};
		this.addListener(listener);

		// Einmal initial ausloesen
		listener.handleEvent(null);
	}
	
	/**
	 * @see de.willuhn.jameica.gui.input.SelectInput#setValue(java.lang.Object)
	 */
	@Override
	public void setValue(final Object currency) {
		super.setValue(currency);
		
		//Kommentar aktualisieren
		final String value = (String) getValue();
		final Currency currencyCode = Currency.find(value);
		setComment(currencyCode == null ? StringUtils.EMPTY : currencyCode.getName());
	}

	/**
	 * Liefert eine Liste der moeglichen W&auml;hrungen.
	 * 
	 * @return Liste der moeglichen W&auml;hrungen.
	 */
	private static List<String> getCodes() {
		final List<String> codes = Currency.codes();
		// codes.add(0,"");
		return codes;
	}
}
