/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.navigation;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.graphics.Image;

import de.frankmuenster.jameica.depot.Settings;
import de.willuhn.datasource.GenericIterator;
import de.willuhn.datasource.GenericObject;
import de.willuhn.datasource.GenericObjectNode;
import de.willuhn.datasource.pseudo.PseudoIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Item;
import de.willuhn.jameica.gui.NavigationItem;
import de.willuhn.jameica.gui.util.SWTUtil;

/**
 * Ein AktienViewer Navigation Item
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class AktienKontoItem implements NavigationItem {

	/**
	 * Das Name-Attribut
	 */
	private final static String NAME_ATTRIBUTE = "name"; //$NON-NLS-1$

	/**
	 * Das Parent Nagivation Item
	 */
	private transient NavigationItem parent;

	/**
	 * Die Aktion
	 */
	private transient Action action;

	/**
	 * Der Navigations Item Text
	 */
	private transient String navitext;

	/**
	 * Die Kinder des Items
	 */
	private transient List<Item> children;

	/**
	 * Das Icon
	 */
	private transient String icon;

	/**
	 * Sind wir enabled
	 */
	private Boolean enabled;

	/**
	 * Erstellt ein AktienKonto Navigation Item.<br/>
	 * <b>Action</b> wird mit <i>null</i> und <b>enabled</b> mit <i>true</i>
	 * initialisiert.
	 * 
	 * @param item
	 *            ist das Elternelement des Navigationsitems
	 * @param navitext
	 *            der Text, der angezeigt wird
	 * @param action
	 *            die Klasse, die das Interface <b>Action</b> implementiert und
	 *            durch das NavigationItem ausgefhrt werden soll.
	 */
	public AktienKontoItem(final NavigationItem item, final String navitext,
			final Action action) {
		this(item, navitext, action, null);
	}

	/**
	 * Erstellt ein AktienKonto Navigation Item.<br/>
	 * <b>Action</b> wird mit <i>null</i> und <b>enabled</b> mit <i>true</i>
	 * initialisiert.
	 * 
	 * @param item
	 *            ist das Elternelement des Navigationsitems
	 * @param navitext
	 *            der Text, der angezeigt wird
	 * @param action
	 *            die Klasse, die das Interface <b>Action</b> implementiert und
	 *            durch das NavigationItem ausgefhrt werden soll.
	 * @param icon
	 *            String des Icon-Bildes des NavigationItems
	 */
	public AktienKontoItem(final NavigationItem item, final String navitext,
			final Action action, final String icon) {
		this(item, navitext, action, icon, true);
	}

	/**
	 * Erstellt ein AktienKonto Navigation Item.<br/>
	 * <b>Action</b> wird mit <i>null</i> und <b>enabled</b> mit <i>true</i>
	 * initialisiert.
	 * 
	 * @param item
	 *            ist das Elternelement des Navigationsitems
	 * @param navitext
	 *            der Text, der angezeigt wird
	 * @param action
	 *            die Klasse, die das Interface <b>Action</b> implementiert und
	 *            durch das NavigationItem ausgefhrt werden soll.
	 * @param icon
	 *            String des Icon-Bildes des NavigationItems
	 * @param enabled
	 *            setzt den Aktiv-Status des Items
	 */
	public AktienKontoItem(final NavigationItem item, final String navitext,
			final Action action, final String icon, final Boolean enabled) {
		this.parent = item;
		this.action = action;
		this.navitext = navitext;
		this.icon = icon;
		this.enabled = enabled;
		children = new ArrayList<Item>();
	}

	/**
	 * @see de.willuhn.jameica.gui.Item#getName()
	 */
	@Override
	public String getName() {
		return navitext;
	}

	/**
	 * @see de.willuhn.jameica.gui.Item#getAction()
	 */
	@Override
	public Action getAction() throws RemoteException {
		return action;
	}

	/**
	 * @see de.willuhn.jameica.gui.Item#addChild(de.willuhn.jameica.gui.Item)
	 */
	@Override
	public void addChild(final Item item) throws RemoteException {
		throw new RemoteException(Settings.getI18n().tr(
				"AktienKontoItem_childNotAllowed")); //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.jameica.gui.Item#isEnabled()
	 */
	@Override
	public boolean isEnabled() throws RemoteException {
		return enabled;
	}

	/**
	 * @see de.willuhn.jameica.gui.Item#setEnabled(boolean, boolean)
	 */
	@Override
	public void setEnabled(final boolean enabled, final boolean recursive)
			throws RemoteException {
		this.enabled = enabled;

	}

	/**
	 * @see de.willuhn.datasource.GenericObjectNode#getChildren()
	 */
	@Override
	public GenericIterator<?> getChildren() throws RemoteException {
		return PseudoIterator.fromArray(children
				.toArray(new AktienKontoItem[children.size()]));
	}

	/**
	 * @see de.willuhn.datasource.GenericObjectNode#getParent()
	 */
	@Override
	public GenericObjectNode getParent() throws RemoteException {
		return parent;
	}

	/**
	 * @see de.willuhn.datasource.GenericObjectNode#getPath()
	 */
	@Override
	public GenericIterator<?> getPath() throws RemoteException {
		@SuppressWarnings("unchecked")
		final List<NavigationItem> list = PseudoIterator.asList(this.parent
				.getPath());
		list.add(this);
		return PseudoIterator.fromArray(list.toArray(new NavigationItem[list
				.size()]));
	}

	/**
	 * @see de.willuhn.datasource.GenericObjectNode#getPossibleParents()
	 */
	@Override
	public GenericIterator<?> getPossibleParents() throws RemoteException {
		throw new RemoteException(Settings.getI18n().tr(
				"AktienKontoItem_notImplemented")); //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.datasource.GenericObjectNode#hasChild(de.willuhn.datasource
	 *      .GenericObjectNode)
	 */
	@Override
	public boolean hasChild(final GenericObjectNode node)
			throws RemoteException {
		return false;
	}

	/**
	 * @see de.willuhn.datasource.GenericObject#equals(de.willuhn.datasource.
	 *      GenericObject)
	 */
	@Override
	public boolean equals(GenericObject item) throws RemoteException { // NOPMD
		boolean isEqual;

		if (item instanceof AktienKontoItem) {
			isEqual = this.getID().equals(item.getID());
		} else {
			isEqual = false;
		}
		return isEqual;
	}

	/**
	 * @see de.willuhn.datasource.GenericObject#getAttribute(java.lang.String)
	 */
	@Override
	public Object getAttribute(final String attribute) throws RemoteException {
		if (!NAME_ATTRIBUTE.equals(attribute)) {
			throw new RemoteException(Settings.getI18n().tr(
					"AktienKontoItem_attributeNotFound")); //$NON-NLS-1$
		}
		return getName();

	}

	/**
	 * @see de.willuhn.datasource.GenericObject#getAttributeNames()
	 */
	@Override
	public String[] getAttributeNames() throws RemoteException {
		return new String[] { NAME_ATTRIBUTE };
	}

	/**
	 * @see de.willuhn.datasource.GenericObject#getID()
	 */
	@Override
	public String getID() {
		return getClass().getName() + "." + getName(); //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.datasource.GenericObject#getPrimaryAttribute()
	 */
	@Override
	public String getPrimaryAttribute() throws RemoteException {
		return NAME_ATTRIBUTE;
	}

	/**
	 * @see de.willuhn.jameica.gui.extension.Extendable#getExtendableID()
	 */
	@Override
	public String getExtendableID() {
		return getID();
	}

	/**
	 * @see de.willuhn.jameica.gui.NavigationItem#getIconOpen()
	 */
	@Override
	public Image getIconOpen() throws RemoteException {
		return SWTUtil.getImage(icon == null ? "jadepot_32x32.png" : icon); //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.jameica.gui.NavigationItem#getIconClose()
	 */
	@Override
	public Image getIconClose() throws RemoteException {
		return SWTUtil.getImage(icon == null ? "jadepot_32x32.png" : icon); //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.jameica.gui.NavigationItem#isExpanded()
	 */
	@Override
	public boolean isExpanded() throws RemoteException {
		return false;
	}

}
