/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.parts;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.rmi.RemoteException;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.kursanalyse.Analysen;
import de.frankmuenster.jameica.depot.kursanalyse.IHandleAddAction;
import de.frankmuenster.jameica.depot.kursanalyse.MethodeAddRemoveListener;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Part das dynamisch Buttons fr noch verfgbare Analysen anzeigt und einen
 * 'Alle Entfernen' Button enthlt.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class AnalyseButtonPart implements Part {

	/**
	 * Unser Parent Composite
	 */
	private transient Composite parent;

	/**
	 * Der Listener
	 */
	private final transient MethodeAddRemoveListener listener;

	/**
	 * Part das dynamisch Buttons fr noch verfgbare Analysen anzeigt und einen
	 * 'Alle Entfernen' Button enthlt.
	 * 
	 * @param listener
	 *            MethodeAddRemoveListener der an die Button angehangen wird.
	 */
	public AnalyseButtonPart(final MethodeAddRemoveListener listener) {
		this.listener = listener;
	}

	/**
	 * @see Part#paint(Composite)
	 */
	@Override
	public void paint(final Composite parent) throws RemoteException {

		this.parent = parent;
		refresh();

	}

	/**
	 * Aktualisiert die angezeigten Buttons.
	 */
	public void refresh() {

		// clear everything
		for (final Control child : parent.getChildren()) {
			child.dispose();
		}

		// Add Buttons
		final Composite buttonArea = new Composite(parent, SWT.NONE);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		gridLayout.makeColumnsEqualWidth = true;
		buttonArea.setLayout(gridLayout);

		final List<Class<?>> methodesClassList = Analysen.getMethods();

		for (final Class<?> methodesClass : methodesClassList) {

			try {
				final Method getName = methodesClass.getMethod("getMethodName", //$NON-NLS-1$
						(Class<?>[]) null);
				final Method getAction = methodesClass.getMethod(
						"getHandleAction", (Class<?>[]) null); //$NON-NLS-1$

				final String name = (String) getName.invoke(null);
				final IHandleAddAction addAction = (IHandleAddAction) getAction
						.invoke(null);
				if (addAction != null) {

					// Analyse schon vorhanden
					final Analysen<?, ?> analysen = Analysen.getInstance();

					if (!analysen.conatinsAnalyseWithLegendTitel(name)) {
						addAction.addListener(listener);
						final AnalyseButton analyseButton = new AnalyseButton( // NOPMD
								name, addAction);
						analyseButton.paint(buttonArea);

					}
				}

			} catch (NoSuchMethodException | SecurityException
					| IllegalAccessException | IllegalArgumentException
					| InvocationTargetException e) {
				Logger.error(Settings.getI18n().tr(
						"AnalyseButtonPart.actionNotFound", e.getMessage())); //$NON-NLS-1$
			} catch (RemoteException e) {
				Logger.error(Settings.getI18n().tr(
						"AnalyseButtonPart.buttonNotFound", e.getMessage())); //$NON-NLS-1$
			}
		}

		// Remove all button
		addRemoveAllButton(buttonArea);

		// Layout berechnen
		buttonArea.layout();
		parent.layout();

	}

	/**
	 * F&uuml;gt den RemoveAll Button hinzu
	 * 
	 * @param buttonArea
	 *            das Composite zu dem den Button hinzuge&uuml;gt werden soll.
	 */
	private void addRemoveAllButton(final Composite buttonArea) {
		final AnalyseButton removeAllBtn = new AnalyseButton(Settings.getI18n()
				.tr("AnalyseButtonPart.removeAllBtn"), //$NON-NLS-1$
				new Action() {
					/**
					 * @see de.willuhn.jameica.gui.Action#handleAction(Object
					 *      context)
					 */
					@Override
					public void handleAction(final Object context)
							throws ApplicationException {
						listener.notifyRemoveAll();
					}
				});

		try {
			removeAllBtn.paint(buttonArea);
		} catch (RemoteException e) {
			Logger.error(Settings.getI18n().tr(
					"AnalyseButtonPart.addButtonError", e.getMessage())); //$NON-NLS-1$
		}
	}

}
