/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.parts;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.gui.input.PeriodChooserInput;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.input.CheckboxInput;

/**
 * Part das verschiedene Werkzeuge zur Diagrammanzeige bietet.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class DiagrammAnzeigeAuswahlPart implements Part {

	/**
	 * Dax anzeuigen
	 */
	private CheckboxInput dax;

	/**
	 * Diagramm als Kerzen zeigen
	 */
	private CheckboxInput highLow;

	/**
	 * Der Zeitauswahlschieber
	 */
	private PeriodChooserInput period;

	/**
	 * Unser Composite in dem wir malen
	 */
	private transient Composite werkzeugArea;

	/**
	 * Standardkonstruktor
	 */
	public DiagrammAnzeigeAuswahlPart() {
		super();
	}

	/**
	 * @return the dax
	 */
	public CheckboxInput getDax() {
		return dax;
	}

	/**
	 * @param dax
	 *            the dax to set
	 */
	public void setDax(final CheckboxInput dax) {
		this.dax = dax;
	}

	/**
	 * @return the highLow
	 */
	public CheckboxInput getHighLow() {
		return highLow;
	}

	/**
	 * @param highLow
	 *            the highLow to set
	 */
	public void setHighLow(final CheckboxInput highLow) {
		this.highLow = highLow;
	}

	/**
	 * @return the period
	 */
	public PeriodChooserInput getPeriod() {
		return period;
	}

	/**
	 * @param period
	 *            the period to set
	 */
	public void setPeriod(final PeriodChooserInput period) {
		this.period = period;
	}

	/**
	 * @return the werkzeugArea
	 */
	public Composite getComposite() {
		return werkzeugArea;
	}

	/**
	 * @see Part#paint(Composite)
	 */
	@Override
	public void paint(final Composite parent) throws RemoteException {

		this.werkzeugArea = new Composite(parent, SWT.NONE);

		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		gridLayout.makeColumnsEqualWidth = false;

		this.werkzeugArea.setLayout(gridLayout);

		final GC graphic = new GC(this.werkzeugArea);
		if (dax != null) {
			final Point daxSize = graphic.textExtent(dax.getName());
			dax.paint(this.werkzeugArea, daxSize.x + 4 + 24);
		}

		final Point highLowSize = graphic.textExtent(highLow.getName());
		graphic.dispose();

		highLow.paint(this.werkzeugArea, highLowSize.x + 15 + 24);
		period.paint(this.werkzeugArea, 300);

	}

}
