/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.parts;

import java.rmi.RemoteException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TabFolder;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierHistory;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.analysegrafik.AnalyseGrafik;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.frankmuenster.jameica.depot.kursanalyse.MethodeAddRemoveListener;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.formatter.CurrencyFormatter;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.jameica.gui.parts.TablePart;
import de.willuhn.jameica.gui.util.ScrolledContainer;
import de.willuhn.jameica.gui.util.TabGroup;

/**
 * Zeit dem Kursverlauf als Diagramm. Bietet verschiedene Anzeigeoptionen, wie
 * DAX Overlay oder Kerzendiagramm an. Oder zeigt die Kursdaten in einer
 * Tabelle. Dazu werden dann noch Analysegrafiken angezeigt, falls vorhanden.
 * 
 * @author munstfr
 * @version 1.0
 */
public class KursHistoryGrafikPart implements Part {

	/**
	 * Einfaches deutsches Datumsformat
	 */
	private final static DateFormat DATEFORMAT = new SimpleDateFormat(
			"dd.MM.yyyy", Locale.GERMAN); //$NON-NLS-1$

	/**
	 * Der Diagramm Titel
	 */
	private String titel;

	/**
	 * Methoden add/remove Listener
	 */
	private final transient MethodeAddRemoveListener listener;

	/**
	 * Die Wertpapierdaten
	 */
	private transient WertpapierHistory wertpapierDaten;

	/**
	 * Die DAX-Daten
	 */
	private transient WertpapierHistory daxDaten;

	/**
	 * Anfangsdatum
	 */
	private Date fromDate;

	/**
	 * Enddatum
	 */
	private Date toDate;

	/**
	 * DAX Overlay anzeigen
	 */
	private boolean showDax;

	/**
	 * Diagramm als Kerzengrafik
	 */
	private boolean showAsCandleStick;

	/**
	 * Hier kommt die Grafik rein, falls zu wenig platz ist.
	 */
	private transient ScrolledContainer scroll;

	/**
	 * Die Analysediagramme, die auch noch angezeigt werden sollen
	 */
	private transient AnalyseGrafik anaG;

	/**
	 * Konstruktor mit Grafik Tietel und den zu benachrichtigenden Listner zum
	 * entfernen von Analysediagrammen.
	 * 
	 * @param titel
	 * @param listener
	 */
	public KursHistoryGrafikPart(final String titel,
			final MethodeAddRemoveListener listener) {
		this.titel = titel;
		this.listener = listener;
	}

	/**
	 * @return titel
	 */
	public String getTitel() {
		return titel;
	}

	/**
	 * @param titel
	 *            das zu setzende Objekt titel
	 */
	public void setTitel(final String titel) {
		this.titel = titel;
	}

	/**
	 * @param wertpapierDaten
	 *            das zu setzende Objekt wertpapierDaten
	 * @param daxHistory
	 */
	public void setKursDaxDaten(final WertpapierHistory wertpapierDaten,
			final WertpapierHistory daxHistory) {
		this.wertpapierDaten = wertpapierDaten;
		this.daxDaten = daxHistory;
	}

	/**
	 * @return the showDax
	 */
	public boolean isShowDax() {
		return showDax;
	}

	/**
	 * @param showDax
	 *            the showDax to set
	 */
	public void setShowDax(final boolean showDax) {
		this.showDax = showDax;
		if (showDax && isShowAsCandleStick()) {
			setShowAsCandleStick(false);
		}
	}

	/**
	 * @param showAsCandleStick
	 *            the showLowHigh to set
	 */
	public void setShowAsCandleStick(final boolean showAsCandleStick) {
		this.showAsCandleStick = showAsCandleStick;
	}

	/**
	 * @return the showAsCandleStick
	 */
	public boolean isShowAsCandleStick() {
		return showAsCandleStick;
	}

	/**
	 * @return the fromDate
	 */
	public Date getFromDate() {
		return this.fromDate;
	}

	/**
	 * @param fromDate
	 *            the fromDate to set
	 */
	public void setFromDate(final Date fromDate) {
		this.setRange(fromDate, this.toDate);
	}

	/**
	 * @return the toDate
	 */
	public Date getToDate() {
		return this.toDate;
	}

	/**
	 * @param toDate
	 *            the toDate to set
	 */
	public void setToDate(final Date toDate) {
		this.setRange(this.fromDate, toDate);
	}

	/**
	 * Setzt den Anzeigezeitraum
	 * 
	 * @param fromDate
	 *            das Startdatum
	 * @param toDate
	 *            das Enddatum
	 */
	public void setRange(final Date fromDate, final Date toDate) {
		this.fromDate = fromDate;
		this.toDate = toDate;

		if (anaG != null) {
			anaG.setRange(fromDate, toDate);
		}
	}

	/**
	 * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void paint(final Composite parent) throws RemoteException {

		// Wir initialisieren die Tabelle.
		final TablePart table = new TablePart(
				wertpapierDaten.getHistoryValues(), null);
		table.addColumn(
				Settings.getI18n().tr("KursHistoryGrafikPart.dateCol"), "date", new Formatter() { //$NON-NLS-1$ //$NON-NLS-2$ 

					/**
					 * Formatter fr Datumsanzeige
					 * 
					 * @param object
					 *            (DATE) das zu formatierende Datum
					 */
					@Override
					public String format(final Object object) {
						String text = StringUtils.EMPTY;
						if (object instanceof Date) {
							synchronized (KursHistoryGrafikPart.class) {
								text = DATEFORMAT.format((Date) object);
							}
						}
						return text;
					}
				});

		table.addColumn(
				Settings.getI18n().tr("KursHistoryGrafikPart.openCol"), "open", new CurrencyFormatter( //$NON-NLS-1$ //$NON-NLS-2$
						null, FormatterUtils.DECIMAL_FORMAT_6.get()));
		table.addColumn(
				Settings.getI18n().tr("KursHistoryGrafikPart.lowCol"), "low", new CurrencyFormatter(null, //$NON-NLS-1$ //$NON-NLS-2$
						FormatterUtils.DECIMAL_FORMAT_6.get()));
		table.addColumn(
				Settings.getI18n().tr("KursHistoryGrafikPart.highCol"), "high", new CurrencyFormatter(null, //$NON-NLS-1$ //$NON-NLS-2$
						FormatterUtils.DECIMAL_FORMAT_6.get()));
		table.addColumn(
				Settings.getI18n().tr("KursHistoryGrafikPart.closeCol"), "close", new CurrencyFormatter( //$NON-NLS-1$ //$NON-NLS-2$
						null, FormatterUtils.DECIMAL_FORMAT_6.get()));
		table.addColumn(Settings.getI18n()
				.tr("KursHistoryGrafikPart.volumeCol"), "volume"); //$NON-NLS-1$ //$NON-NLS-2$
		table.addColumn(
				Settings.getI18n().tr("KursHistoryGrafikPart.adjustedCol"), "adj-closed", //$NON-NLS-1$ //$NON-NLS-2$
				new CurrencyFormatter(null, FormatterUtils.DECIMAL_FORMAT_6.get()));
		table.setRememberOrder(true);
		table.setRememberColWidths(true);

		final TabFolder folder = new TabFolder(parent, SWT.NONE);
		folder.setLayoutData(new GridData(GridData.FILL_BOTH));

		final TabGroup grafiken = new TabGroup(folder, Settings.getI18n().tr(
				"KursHistoryGrafikPart.chartTab"), //$NON-NLS-1$
				false, 1);
		final GridLayout gridLayout = new GridLayout(1, false);
		grafiken.getComposite().setLayout(gridLayout);

		final TabGroup tabGroup = new TabGroup(folder, Settings.getI18n().tr(
				"KursHistoryGrafikPart.tableTab")); //$NON-NLS-1$
		table.paint(tabGroup.getComposite());

		scroll = new ScrolledContainer(grafiken.getComposite(), 1);

		anaG = new AnalyseGrafik(listener);
		scroll.addPart(anaG);

		anaG.setTitle(titel);
		anaG.setKursDaxDaten(wertpapierDaten, daxDaten);
		anaG.setRange(fromDate, toDate);

		refresh();

	}

	/**
	 * Aktualisiert die Anzeige
	 */
	public void refresh() {

		anaG.setShowDax(showDax);
		anaG.setShowAsCandleStick(showAsCandleStick);
		anaG.reload();

		scroll.update();
	}

}
