/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.parts;

import java.rmi.RemoteException;
import java.text.DecimalFormat;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.TableItem;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.gui.parts.column.OrderTotal;
import de.frankmuenster.jameica.depot.gui.parts.column.OrderTotalInStandartWaehrung;
import de.willuhn.datasource.GenericIterator;
import de.willuhn.datasource.GenericObject;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.formatter.CurrencyFormatter;
import de.willuhn.jameica.gui.formatter.DateFormatter;
import de.willuhn.jameica.gui.formatter.TableFormatter;
import de.willuhn.jameica.gui.parts.TablePart;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.SWTUtil;
import de.willuhn.logging.Logger;

/**
 * Ein Tabelle mit den Ordern eines Wertpapieres.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class OrderListePart extends TablePart implements Part {

	/**
	 * Pattern f&uuml;r die W&auml;hrungsdarstellung
	 */
	private final static String PATTERN = "###,###,##0.00"; //$NON-NLS-1$

	/**
	 * Konstruktor mit Angabe der Orderliste und der Action fr den Doppelklick.
	 * 
	 * @param list
	 * @param action
	 * @throws RemoteException
	 */
	public OrderListePart(final GenericIterator<?> list, final Action action)
			throws RemoteException {
		super(list, action);

		String waehrung = "%"; //$NON-NLS-1$

		// suche Wertpapier
		if (list.hasNext()) {
			final GenericObject nextListObject = list.next();
			if (nextListObject instanceof OrderHistory) {
				waehrung = ((OrderHistory) nextListObject).getAktie()
						.getCurrency();
			}
		}

		final boolean showWaehrungsCol = !waehrung.equals(Settings
				.getDepotWaehrung());

		final DecimalFormat decimalFormat = new DecimalFormat(PATTERN);
		final CurrencyFormatter currencyFormatter = new CurrencyFormatter(
				waehrung, decimalFormat);
		final DateFormatter dateFormatter = new DateFormatter();

		setRememberColWidths(true);
		setRememberOrder(true);
		setRememberState(true);
		setSummary(false);

		setFormatter(new MyTableFormatter());

		// addColumn("ID", "id");
		addColumn(
				Settings.getI18n().tr("OrderListePart.flagCol"), "orderAmount"); //$NON-NLS-1$ //$NON-NLS-2$
		addColumn(
				Settings.getI18n().tr("OrderListePart.dateCol"), "orderDate", dateFormatter); //$NON-NLS-1$ //$NON-NLS-2$
		addColumn(
				Settings.getI18n().tr("OrderListePart.amountCol"), "orderAmount"); //$NON-NLS-1$ //$NON-NLS-2$
		addColumn(
				Settings.getI18n().tr("OrderListePart.tradeValueCol"), "orderPrice", //$NON-NLS-1$ //$NON-NLS-2$
				currencyFormatter);
		if (showWaehrungsCol) {
			addColumn(new OrderTotalInStandartWaehrung());
		} else {
			addColumn(new OrderTotal());
		}
		addColumn(
				Settings.getI18n().tr("OrderListePart.additionalCostsCol"), "additionalCosts", //$NON-NLS-1$ //$NON-NLS-2$
				currencyFormatter);
		if (showWaehrungsCol) {
			addColumn(
					Settings.getI18n().tr(
							"OrderListePart.inStandardCurrency", Settings.getDepotWaehrung()), //$NON-NLS-1$
					"additionalCostsInMainCurrency"); //$NON-NLS-1$
		}
		addColumn(Settings.getI18n().tr("OrderListePart.commentCol"), "comment"); //$NON-NLS-1$ //$NON-NLS-2$

	}

	/**
	 * Einen netten Formatierer f&uuml;r unser Tabelle.
	 * 
	 * @author munstfr
	 * @version 1.0
	 * 
	 */
	private class MyTableFormatter implements TableFormatter {

		/**
		 * Standardkonstruktor
		 */
		public MyTableFormatter() {
			// Absichtlich leer
		}

		/**
		 * @see TableFormatter#format(TableItem)
		 */
		@Override
		public void format(final TableItem item) {

			if (item.getData() instanceof OrderHistory) {
				final OrderHistory orderHistory = (OrderHistory) item.getData();

				try {

					item.setText(0, StringUtils.EMPTY);
					if (orderHistory.getOrderAmount() < 0) {
						item.setForeground(Color.SUCCESS.getSWTColor());
						// Zusatzkosten nicht einfrben
						item.setForeground(5, Color.FOREGROUND.getSWTColor());
						item.setImage(0, SWTUtil.getImage("list-remove.png")); //$NON-NLS-1$
					} else {
						item.setForeground(Color.FOREGROUND.getSWTColor());
						item.setImage(0, SWTUtil.getImage("list-add.png")); //$NON-NLS-1$
					}
				} catch (RemoteException e) {
					Logger.error(
							Settings.getI18n().tr(
									"OrderListePart.formatingError"), e); //$NON-NLS-1$
				}
			}
		}
	}

}
