/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.parts;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Text;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.StockInfoBean;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceAPI;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceException;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceExceptionType;
import de.frankmuenster.jameica.depot.gui.utils.SWTFontUtils;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Part das es erlaubt nach einem Wertpapier-Symbol zu suchen. Hier via YAHOO.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class SymbolSearchPart implements Part {

	/**
	 * Platzhalter im Suchergebnis 3 Zeichen
	 */
	private final static String SPACE_3 = "   "; //$NON-NLS-1$

	/**
	 * Platzhalter im Suchergebnis 10 Zeichen
	 */
	private final static String SPACE_10 = "          "; //$NON-NLS-1$

	/**
	 * Platzhalter im Suchergebnis 20 Zeichen
	 */
	private final static String SPACE_20 = "                    "; //$NON-NLS-1$

	/**
	 * Suchtext-Eingabefeld
	 */
	private transient Text searchInput;

	/**
	 * Die Liste mit den Suchergebnissen
	 */
	private transient List searchResult;

	/**
	 * Das Composite in das wird das Suchpart malen.
	 */
	// private transient Composite inner;

	/**
	 * Liste mit StockInfo's
	 */
	private transient java.util.List<StockInfoBean> resultList = new ArrayList<StockInfoBean>();

	/**
	 * Unsere Listeners fr SearchResultSelected
	 */
	private final transient Set<SearchResultSelectListener> listeners = new CopyOnWriteArraySet<SearchResultSelectListener>();

	/**
	 * Fuegt eine Listener hinzu
	 * 
	 * @param listener
	 *            SearchResultSelectListener der hinzugefuegt werden soll
	 */
	public final void addListener(final SearchResultSelectListener listener) {
		listeners.add(listener);
	}

	/**
	 * Entfernd einen Listener aus der Liste
	 * 
	 * @param listener
	 *            SearchResultSelectListener der entfernt werden soll
	 */
	public final void removeListener(final SearchResultSelectListener listener) {
		listeners.remove(listener);
	}

	/**
	 * 
	 * Benachrichtigen der registrieretn Listeners mit bergabe dem gefundenen
	 * QuoteLookupInfoBean Element
	 * 
	 * @param index
	 */
	private final void notifyListeners(final int index) {
		for (final SearchResultSelectListener listener : listeners) {

			final StockInfoBean stockInfoBean = resultList.get(index);
			listener.searchResultSelected(stockInfoBean);
		}
	}

	/**
	 * Standardkonstruktor mit Angabe eines Listeners
	 * 
	 * @param listener
	 *            der zu benachrichtigende Listener
	 */
	public SymbolSearchPart(final SearchResultSelectListener listener) {
		addListener(listener);

	}

	/**
	 * @see Part#paint(Composite)
	 */
	@Override
	public void paint(final Composite parent) throws RemoteException {

		if (parent == null) {
			Logger.error(Settings.getI18n().tr(
					"SymbolSearchPart.logParentIsNull")); //$NON-NLS-1$
		}

		final GridLayout gridLayout = new GridLayout(2, false);
		parent.setLayout(gridLayout);

		final Label nameLabel = new Label(parent, SWT.NONE);
		nameLabel.setText(Settings.getI18n().tr("SymbolSearchPart.lblName")); //$NON-NLS-1$

		// searchInput = new Text(parent, SWT.BORDER);
		searchInput = GUI.getStyleFactory().createText(parent);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		searchInput.setLayoutData(gridData);
		searchInput.setText(StringUtils.EMPTY);

		searchInput.addKeyListener(new SearchInputKeyListener());

		final Label searchResultLabel = new Label(parent, SWT.NONE);
		searchResultLabel.setText(Settings.getI18n().tr(
				"SymbolSearchPart.lblSearchResults")); //$NON-NLS-1$
		gridData = new GridData();
		gridData.verticalAlignment = SWT.BEGINNING;
		// gridData.horizontalSpan = 2;
		searchResultLabel.setLayoutData(gridData);

		searchResult = new List(parent, SWT.BORDER | SWT.SINGLE | SWT.V_SCROLL);
		gridData = new GridData(GridData.FILL_BOTH);
		// gridData.horizontalSpan = 2;
		gridData.horizontalAlignment = SWT.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = SWT.FILL;
		gridData.grabExcessVerticalSpace = true;
		searchResult.setLayoutData(gridData);
		Font font = GUI.getDisplay().getSystemFont();
		try {
			font = SWTFontUtils.getMonospacedFont();
		} catch (AssertionError | RuntimeException ex) { //NOPMD
			Logger.warn(Settings.getI18n().tr("SymbolSearchPart.monospacedFontNotFound")); //$NON-NLS-1$
			Application
			.getMessagingFactory()
			.sendMessage(
					new StatusBarMessage(
							Settings.getI18n()
									.tr("SymbolSearchPart.monospacedFontNotFoundMsg"), //$NON-NLS-1$
							StatusBarMessage.TYPE_INFO));
		}

		final FontData fontData = font.getFontData()[0];
		fontData.setHeight(8);
		font = new Font(parent.getDisplay(), fontData);
		searchResult.setFont(font);

		searchResult.add(StringUtils.EMPTY);
		searchResult.add(StringUtils.EMPTY);
		searchResult.add(StringUtils.EMPTY);
		searchResult.add(StringUtils.EMPTY);
		searchResult.add(StringUtils.EMPTY);
		searchResult.add(StringUtils.EMPTY);
		searchResult.add(StringUtils.EMPTY);

		searchResult.addSelectionListener(new SeachResultSelectionListener());

	}

	private String formatListEntry(final StockInfoBean stockInfoBean) {

		final StringBuffer result = new StringBuffer();
		StringBuffer help = new StringBuffer(stockInfoBean.getSymbol())
				.append(SPACE_10);
		result.append(help.substring(0, 10)).append(SPACE_3);
		help = new StringBuffer(stockInfoBean.getName()).append(SPACE_20);
		result.append(help.substring(0, 20)).append(SPACE_3);
		help = new StringBuffer(stockInfoBean.getExchange()).append(SPACE_10);
		result.append(help.substring(0, 10)).append(SPACE_3);
		help = new StringBuffer(stockInfoBean.getType()).append(SPACE_10);
		result.append(help.substring(0, 10)).append(SPACE_3);

		return result.toString();

	}

	/**
	 * Auswahllistener der anzeigt, dass ein Suchergebniss angeklickt wurde.
	 * 
	 * @author munstfr
	 * @version 1.0
	 * 
	 */
	private class SeachResultSelectionListener implements SelectionListener {

		/**
		 * Standartkonstruktor
		 */
		public SeachResultSelectionListener() {
			// absichtlich leer
		}

		/**
		 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org
		 *      .eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetDefaultSelected(final SelectionEvent arg0) {
			// nichts zu tun hier

		}

		/**
		 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse
		 *      .swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(final SelectionEvent arg0) {
			Logger.warn(Settings
					.getI18n()
					.tr("SymbolSearchPart.logSelectedItem", Integer.toString(searchResult.getSelectionIndex()))); //$NON-NLS-1$
			notifyListeners(searchResult.getSelectionIndex());
		}

	}

	/**
	 * Listner der Tastenanschlge verarbeitet.
	 * 
	 * @author munstfr
	 * @version 1.0
	 * 
	 */
	private class SearchInputKeyListener implements KeyListener {

		/**
		 * Minimale Lnge des Suchwortes, bevor Anfragen an Yahoo gesendet
		 * werden.
		 */
		private final static int MIN_WORD_LENGTH = 3;

		/**
		 * Standartkonstruktor
		 */
		public SearchInputKeyListener() {
			// absichtlich leer
		}

		/**
		 * @see KeyListener#keyPressed(KeyEvent)
		 */
		@Override
		public void keyPressed(final KeyEvent arg0) {
			// nichts zu tun hier.
		}

		/**
		 * @see KeyListener#keyReleased(KeyEvent)
		 */
		@Override
		public void keyReleased(final KeyEvent arg0) {

			final String search = searchInput.getText();
			if (search.length() > MIN_WORD_LENGTH) {

				try {
					resultList = YahooFinanceAPI.getInstance().lookUpQuote(
							search);
				} catch (YahooFinanceException e) {
					if (e.getType().equals(
							YahooFinanceExceptionType.DATA_NOT_FOUND)) {
						Logger.error(Settings.getI18n().tr(
								"SymbolSearchPart.dataNotFound", //$NON-NLS-1$
								e.getMessage()));
						Application
								.getMessagingFactory()
								.sendMessage(
										new StatusBarMessage(
												Settings.getI18n()
														.tr("SymbolSearchPart.quoteLookupFailedMsg"), //$NON-NLS-1$
												StatusBarMessage.TYPE_ERROR));
					}
				}
				searchResult.removeAll();
				if (resultList != null) {
					for (int i = 0; i < resultList.size(); i++) {
						searchResult.add(formatListEntry(resultList.get(i)));
					}
				}
			} else {
				searchResult.removeAll();
			}
		}
	}

	/**
	 * Listener zum Anzeigen, dass ein ein Suchergebnis angeklickt wurde.
	 * 
	 * @author munstfr
	 * @version 1.0
	 */
	public interface SearchResultSelectListener {

		/**
		 * Gibt den das angeklickte yahooQuoteLookupInfo elemet zurck
		 * 
		 * @param stockInfoBean
		 */
		void searchResultSelected(StockInfoBean stockInfoBean);
	}

}
