/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.parts.grafiken;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;

/**
 * Extended ArrayList mit DateValue Paaren, die eine Kurshistorie abbilden.
 * Zus&auml;tzliche Funktionen liefert Listen nur bestehende aus dem datum, bzw.
 * den Werten.
 * 
 * @author munstfr
 * @version 1.0
 */
public class KursDaten extends ArrayList<DateValuePair> {

	/**
	 * Serial UID
	 */
	private static final long serialVersionUID = 5029613231402146456L;

	/**
	 * Standardkonstrukto
	 */
	public KursDaten() {
		super();
	}

	/**
	 * Liefert eine Liste mit allen Daten
	 * 
	 * @return Liste mit allen Daten
	 */
	public List<Date> getDateList() {
		final List<Date> dateList = new ArrayList<Date>();

		for (final DateValuePair resultValue : this) {
			dateList.add(resultValue.getDate());
		}
		return dateList;
	}

	/**
	 * Liefert eine Liste mit allen Werten
	 * 
	 * @return Liste mit allen Werten
	 */
	public List<Double> getValueList() {
		final List<Double> valueList = new ArrayList<Double>();

		for (final DateValuePair resultValue : this) {
			valueList.add(resultValue.getValue());
		}
		return valueList;
	}

	/**
	 * Erstellt ein TimeSeries Objekt aus eine Kursdaten Objekt
	 * 
	 * @param titel
	 * @return TimeSeries Objekt mit den Kursdaten
	 */
	public TimeSeries getTimeSeriesFromKursDaten(final String titel) {

		final TimeSeries serieAnalyse = new TimeSeries(titel);

		for (final DateValuePair value : this) {
			final Date date = value.getDate();
			final Day day = new Day(date); // NOPMD
			serieAnalyse.add(day, value.getValue());
		}

		return serieAnalyse;
	}
}
