/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.utils;

import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Locale;

import de.willuhn.jameica.gui.formatter.CurrencyFormatter;

/**
 * Utility-Klasse f&uml;r verschiedene Formatter. Diese werden Thread-Safe
 * geliefert und m&uuml;ssen deshalb mit <i>.get()</i> benutzt werden.
 * 
 * @author munstfr
 * @version 1.0
 */
@SuppressWarnings("PMD.LongVariable")
public final class FormatterUtils {

	/**
	 * Dezimal-Format mit genau 2 Nachkommastellen
	 */
	public static final ThreadLocal<DecimalFormat> DECIMAL_FORMAT = new ThreadLocal<DecimalFormat>() {
		/**
		 * @see ThreadLocal#initialValue()
		 */
		@Override
		protected DecimalFormat initialValue() {
			return new DecimalFormat("###,###,##0.00"); //$NON-NLS-1$
		}
	};

	/**
	 * Dezimal-Format mit min. 2 bis zu 3 Nachkommastellen
	 */
	public static final ThreadLocal<DecimalFormat> DECIMAL_FORMAT_3 = new ThreadLocal<DecimalFormat>() {
		/**
		 * @see ThreadLocal#initialValue()
		 */
		@Override
		protected DecimalFormat initialValue() {
			return new DecimalFormat("###,###,##0.00#"); //$NON-NLS-1$
		}
	};

	/**
	 * Dezimal-Format mit min. 2 bis zu 6 Nachkommastellen
	 */
	public static final ThreadLocal<DecimalFormat> DECIMAL_FORMAT_6 = new ThreadLocal<DecimalFormat>() {
		/**
		 * @see ThreadLocal#initialValue()
		 */
		@Override
		protected DecimalFormat initialValue() {
			return new DecimalFormat("###,###,##0.00######"); //$NON-NLS-1$
		}
	};

	/**
	 * W&auml;hrungs-Format mit genau 2 Nachkommastellen
	 */
	public final static ThreadLocal<CurrencyFormatter> CURRENCY_FORMAT = new ThreadLocal<CurrencyFormatter>() {
		/**
		 * @see ThreadLocal#initialValue()
		 */
		@Override
		protected CurrencyFormatter initialValue() {
			return new CurrencyFormatter(null, DECIMAL_FORMAT.get());
		}
	};

	/**
	 * W&auml;hrungs-Format mit bis zu 3 Nachkommastellen
	 */
	public final static ThreadLocal<CurrencyFormatter> CURRENCY_FORMAT_3 = new ThreadLocal<CurrencyFormatter>() {
		/**
		 * @see ThreadLocal#initialValue()
		 */
		@Override
		protected CurrencyFormatter initialValue() {
			return new CurrencyFormatter(null, DECIMAL_FORMAT_3.get());
		}
	};

	/**
	 * W&auml;hrungs-Format mit bis zu 6 Nachkommastellen
	 */
	public final static ThreadLocal<CurrencyFormatter> CURRENCY_FORMAT_6 = new ThreadLocal<CurrencyFormatter>() {
		/**
		 * @see ThreadLocal#initialValue()
		 */
		@Override
		protected CurrencyFormatter initialValue() {
			return new CurrencyFormatter(null, DECIMAL_FORMAT_6.get());
		}
	};

	/**
	 * Deutsches Datumsformat "dd.MM.yyyy"
	 */
	public static final ThreadLocal<DateFormat> GERMAN_DATE_FORMAT = new ThreadLocal<DateFormat>() {
		/**
		 * @see ThreadLocal#initialValue()
		 */
		@Override
		protected DateFormat initialValue() {
			return new SimpleDateFormat("dd.MM.yyyy", Locale.GERMAN); //$NON-NLS-1$
		}
	};

	/**
	 * Einfaches Zeitformat "h:mm"
	 */
	public static final ThreadLocal<DateFormat> SIMPLE_TIME_FORMAT = new ThreadLocal<DateFormat>() {
		/**
		 * @see ThreadLocal#initialValue()
		 */
		@Override
		protected DateFormat initialValue() {
			return new SimpleDateFormat("h:mma", Locale.GERMAN); //$NON-NLS-1$
		}
	};

	/**
	 * Standardkonstruktor
	 */
	private FormatterUtils() {
		// absichlich leer
	}

}
