/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.views;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.controller.AnalysenKontroller;
import de.frankmuenster.jameica.depot.kursanalyse.Analysen;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.Headline;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * @author munstfr
 * 
 */
public class AnalysenView extends AbstractView {

	// private View view;
	/**
	 * Der AnalysenView-Kontroller
	 */
	private transient AnalysenKontroller control;

	/**
	 * Standardkonstruktor
	 */
	public AnalysenView() {
		super();
	}

	/**
	 * @see de.willuhn.jameica.gui.AbstractView#bind()
	 */
	@SuppressWarnings("unused")
	@Override
	public void bind() throws RemoteException {

		// draw the title
		GUI.getView().setTitle(Settings.getI18n().tr("AnalysenView.viewTitle")); //$NON-NLS-1$

		// Instantiate controller
		control = new AnalysenKontroller(this);

		final Container container = new SimpleContainer(getParent());

		// layout with 2 columns
		final ColumnLayout columns = new ColumnLayout(container.getComposite(),
				2);

		final Container left = new SimpleContainer(columns.getComposite());
		final Container right = new SimpleContainer(columns.getComposite(),
				true);

		// left side
		left.addPart(control.getZeitauswahl());
		left.addPart(control.getGrafikWerkzeuge());

		// Rechte Seite
		right.addHeadline(Settings.getI18n().tr("AnalysenView.analysis")); //$NON-NLS-1$
		right.addPart(control.getAnalyseButtons());

		// zum Schluss die Grafik
		new Headline(getParent(), Settings.getI18n().tr("AnalysenView.Chart")); //$NON-NLS-1$
		control.getGrafik().paint(getParent());

	}

	/**
	 * @see de.willuhn.jameica.gui.AbstractView#unbind()
	 */
	@Override
	public void unbind() throws ApplicationException {
		super.unbind();
		// Analysen leeren
		Analysen.getInstance().clear();
		// Kontroller lschen
		control = null; // NOPMD

	}

}
