/*
 * Copyright 2016 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.views;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.TabFolder;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.controller.SettingsController;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.gui.util.TabGroup;
import de.willuhn.util.ApplicationException;

/**
 * View fr die Einstellungen
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class SettingsView extends AbstractView {

	/**
	 * Der Settings-Kontrolleur
	 */
	private transient SettingsController control;

	/**
	 * Standardkonstruktor
	 */
	public SettingsView() {
		super();
	}

	/**
	 * @see de.willuhn.jameica.gui.AbstractView#bind()
	 */
	@Override
	public void bind() throws RemoteException {

		GUI.getView().setTitle(Settings.getI18n().tr("SettingsView.viewTitle")); //$NON-NLS-1$

		// Kontrolleur erstellen
		control = new SettingsController(this);

		final GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.verticalSpacing = 10;

		final TabGroup grundEinstellung = new TabGroup(getTabFolder(), Settings
				.getI18n().tr("SettingsView.tabBasicSettings")); //$NON-NLS-1$

		final ColumnLayout columnLayout = new ColumnLayout(
				grundEinstellung.getComposite(), 2);
		final SimpleContainer leftContainer = new SimpleContainer(
				columnLayout.getComposite());
		final SimpleContainer rightContainer = new SimpleContainer(
				columnLayout.getComposite());

		leftContainer.addHeadline(Settings.getI18n().tr(
				"SettingsView.hlCommonSettings")); //$NON-NLS-1$
		if (Settings.isHibiscusInstalled()) {
			leftContainer.addLabelPair(
					Settings.getI18n().tr("SettingsView.lblShowHibiscus"), //$NON-NLS-1$
					control.getShowHibiscusSummery());
		}
		leftContainer.addText("     ", false); //$NON-NLS-1$

		rightContainer.addText("       ", false); //$NON-NLS-1$

		final ButtonArea buttonArea = new ButtonArea();
		buttonArea.addButton(
				Settings.getI18n().tr("SettingsView.btnSave"), new Action() { //$NON-NLS-1$
					/**
					 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
					 */
					@Override
					public void handleAction(final Object context)
							throws ApplicationException {
						control.speichern();
					}
				}, null, true, "document-save.png"); //$NON-NLS-1$
		buttonArea.paint(getParent());

	}

	/**
	 * @see de.willuhn.jameica.gui.AbstractView#unbind()
	 */
	@Override
	public void unbind() throws ApplicationException {
		super.unbind();
		control = null; // NOPMD
	}

	/**
	 * Liefert den Tab-Folder, in dem die einzelnen Module der Einstellungen
	 * untergebracht sind.
	 * 
	 * @return der Tab-Folder.
	 */
	private TabFolder getTabFolder() {

		final TabFolder folder = new TabFolder(getParent(), SWT.NONE);
		folder.setLayoutData(new GridData(GridData.FILL_BOTH));
		return folder;
	}

}
