/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.kursanalyse.EMA; // NOPMD

import org.eclipse.swt.graphics.Color;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.gui.utils.ColorUtils;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractHandleAddAction;
import de.frankmuenster.jameica.depot.kursanalyse.IHandleAddAction;
import de.frankmuenster.jameica.depot.kursanalyse.IKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseLineStyle;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseParameter;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseTyp;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * EMA AddAction.
 * 
 * @author munstfr
 * @version 1.0
 */
public class EmaHandleAddAction extends AbstractHandleAddAction implements
		IHandleAddAction {

	/**
	 * Standardkonstruktor
	 */
	public EmaHandleAddAction() {
		super();
	}

	/**
	 * @see de.willuhn.jameica.gui.Action#handleAction(Object context)
	 */
	@Override
	public void handleAction(final Object context) throws ApplicationException {
		Logger.info(Settings.getI18n().tr("EmaHandleAddAction.logAdd")); //$NON-NLS-1$
		handleAddEMA();

	}

	private void handleAddEMA() {

		final EmaAddDialog dialog = new EmaAddDialog();
		int order = 0;
		try {
			order = (Integer) dialog.open();
		} catch (OperationCanceledException oce) {
			Logger.info(Settings.getI18n().tr(
					"EmaHandleAddAction.logAddCanceled")); //$NON-NLS-1$
		} catch (Exception e) { // NOPMD
			Logger.error(Settings.getI18n()
					.tr("EmaHandleAddAction.logAddError"), e); //$NON-NLS-1$
		}

		if (order > 0) {

			final KursAnalyseParameter<Integer> parameter = new KursAnalyseParameter<Integer>(
					order);
			final IKursAnalyse<Integer, KursDaten[]> ema = new ExponentialMovingAverage();
			ema.setKursAnalyseParameter(parameter);
			ema.setType(KursAnalyseTyp.OVERLAY_LINE);
			ema.setLineCount(1);
			final Color color = ColorUtils.getRandomColor();
			ema.setColor(color);
			final KursAnalyseLineStyle lineStyle = KursAnalyseLineStyle.FULL;
			ema.setLineStyle(lineStyle);

			// Call listener
			notifyListeners(ema);
		}
	}
}
