/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.kursanalyse;

import java.util.Date;

import org.eclipse.swt.graphics.Color;

import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierHistory;

/**
 * Interface einer Analysemethode.
 * 
 * @param <P>
 *            Der Type des KursAnalyseParameters
 * @param <R>
 *            Der Ergebnistyp. Darf nur KursDaten[] oder XYPLot sein.
 * 
 * @author munstfr
 * @version 1.0
 */
public interface IKursAnalyse<P, R> {

	/**
	 * Setzt die historischen WP-Werte.
	 * 
	 * @param wertpapierHistory
	 */
	void setWertpapierHistory(WertpapierHistory wertpapierHistory);

	/**
	 * Setzte den KursAnalyseParameter
	 * 
	 * @param parameter
	 */
	void setKursAnalyseParameter(KursAnalyseParameter<P> parameter);

	/**
	 * Liefert den Typ der Analyse (Overlay-Linie/-Bar, eigenst&auml;ndiger
	 * Chart.
	 * 
	 * @return der KursAnalyseTyp
	 */
	KursAnalyseTyp getType();

	/**
	 * Setzt den Typ der Analyse (Overlay-Linie/-Bar, eigenst&auml;ndiger Chart.
	 * 
	 * @param type
	 *            der KursAnalyseTyp
	 */
	void setType(KursAnalyseTyp type);

	/**
	 * Liefert den Chart-Legenden-Titel.
	 * 
	 * @return den Chart-Legenden-Titel
	 */
	String getLegendTitel();

	/**
	 * Liefert die Farben.
	 * 
	 * @return die Farben
	 */
	Color[] getColor();

	/**
	 * Setzt die Farben.
	 * 
	 * @param color
	 *            das zu setzende Farb-Array
	 */
	void setColor(Color... color);

	/**
	 * Liefert den Linien-Style, <I>FULL</i> oder <i>DOTTED</i>.
	 * 
	 * @return den KursAnalyseLineStyle
	 */
	KursAnalyseLineStyle[] getLineStyle();

	/**
	 * Setzt den Linien-Style, <I>FULL</i> oder <i>DOTTED</i>.
	 * 
	 * @param lineStyle
	 *            den KursAnalyseLineStyle
	 */
	void setLineStyle(KursAnalyseLineStyle... lineStyle);

	/**
	 * Setzt die Anzahl der Linien im Kurstyp OVERLAY
	 * 
	 * @return Anzahl Linien
	 */
	int getLineCount();

	/**
	 * Setzt die Anzahl an Kurslinien im Kurstyp OVERLAY
	 * 
	 * @param count
	 *            die Anzhal an Linien
	 */
	void setLineCount(int count);

	/**
	 * Liefert das Ergebnis der Analyse f&uuml;r den angegebenen Zeitraum.
	 * Entweder ein Array aus <i>KursDaten[]</i> oder eine Chart als
	 * <i>XYPlot</i>.
	 * 
	 * @param fromDate
	 *            das Anfangsdatum
	 * @param toDate
	 *            das Enddatum
	 * @return entweder <i>KursDaten[]</i>, falls der Typ einer der
	 *         <i>OVERLAY</i>-Typen ist, oder <i>XYPLot</i>, falls der Typ
	 *         <i>SEPERATE_GRAPH</i> ist.
	 */
	R getResult(Date fromDate, Date toDate);

}