/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.kursanalyse.MOM; // NOPMD 

import java.awt.Font;
import java.util.Date;

import org.jfree.chart.annotations.XYTitleAnnotation;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.SegmentedTimeline;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.willuhn.jameica.gui.Action;
import de.willuhn.logging.Logger;

/**
 * Momentum Methode
 * 
 * @author munstfr
 * @version 1.0
 */
public class Momentum extends AbstractKursAnalyse<MomentumParameter, XYPlot> {

	/**
	 * Methodenname
	 */
	private final transient static String NAME = "Momentum"; //$NON-NLS-1$

	/**
	 * Werte der Serie
	 */
	private transient TimeSeries serieMOM;

	/**
	 * Der Rendere
	 */
	private transient XYItemRenderer rendererMOM;

	/**
	 * Standardkonstruktor
	 */
	public Momentum() {
		super();
	}

	/**
	 * Liefert den Methodennamen
	 * 
	 * @return der Methodennamen
	 */
	public static String getMethodName() {
		return NAME;
	}

	/**
	 * Liefert die AddAction
	 * 
	 * @return die MomentumHandleAddAction
	 */
	public static Action getHandleAction() {
		return new MomentumHandleAddAction();
	}

	/**
	 * @see AbstractKursAnalyse#getLegendTitel()
	 */
	@Override
	public String getLegendTitel() {
		final MomentumParameter para = parameter.getValue();
		final int period = para.getPeriod();
		return "Momentum (" + period + ")"; //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * 
	 * @see AbstractKursAnalyse#getResult(java.util.Date, java.util.Date)
	 */
	@Override
	public XYPlot getResult(final Date fromDate, final Date toDate) {

		if (anaG == null) {

			final DateAxis dateAxisKurs = new DateAxis(Settings.getI18n().tr(
					"Momentum.axisDate")); //$NON-NLS-1$
			final NumberAxis rangeAxisKurs = new NumberAxis(Settings.getI18n()
					.tr("Momentum.axisNumber")); //$NON-NLS-1$
			rangeAxisKurs.setAutoRangeIncludesZero(false);
			anaG = new XYPlot(null, dateAxisKurs, rangeAxisKurs, null);
			((DateAxis) anaG.getDomainAxis()).setTimeline(SegmentedTimeline
					.newMondayThroughFridayTimeline());
			anaG.setRangeAxisLocation(AxisLocation.BOTTOM_OR_LEFT);

			// XYAreaRenderer rendererMACD = new XYAreaRenderer();
			rendererMOM = new StandardXYItemRenderer();
			rendererMOM.setSeriesPaint(0, java.awt.Color.BLACK);

			final LegendTitle legendTitle = new LegendTitle(anaG);
			legendTitle.setItemFont(new Font("Dialog", Font.PLAIN, 9)); //$NON-NLS-1$
			legendTitle.setBackgroundPaint(new java.awt.Color(200, 200, 255,
					100));
			legendTitle.setFrame(new BlockBorder(java.awt.Color.white));
			legendTitle.setPosition(RectangleEdge.BOTTOM);
			final RectangleInsets padding = new RectangleInsets(4, 2, 0, 10);
			legendTitle.setItemLabelPadding(padding);
			final XYTitleAnnotation titleAnnotation = new XYTitleAnnotation(
					1.0, 0.00, legendTitle, RectangleAnchor.BOTTOM_RIGHT);
			titleAnnotation.setMaxWidth(0.48);
			anaG.addAnnotation(titleAnnotation);
		}

		if (serieMOM == null) {
			callculate();
		}

		final Day fromDay = new Day(fromDate);
		final Day toDay = new Day(toDate);

		try {
			final TimeSeriesCollection tmpMom = new TimeSeriesCollection(
					serieMOM.createCopy(fromDay, toDay));
			anaG.setDataset(0, tmpMom);
			anaG.setRenderer(0, rendererMOM);

		} catch (CloneNotSupportedException e) {
			Logger.error(Settings.getI18n().tr("Momentum.createTsDataError"), //$NON-NLS-1$
					e);
		}

		return anaG;

	}

	/**
	 * Nun mal rechnen
	 */
	private void callculate() {

		final int period = parameter.getValue().getPeriod();

		serieMOM = new TimeSeries(getLegendTitel());

		Date date = closeDaten.get(0).getDate();
		Day day = new Day(date);

		serieMOM.add(day, 0.0);
		for (int i = 1; i < closeDaten.size(); i++) {

			double newValue = Double.NaN;

			final double value = closeDaten.get(i).getValue();
			date = closeDaten.get(i).getDate();
			day = new Day(date); // NOPMD
			if (i >= period) {
				newValue = value - closeDaten.get(i - period).getValue();
			}
			serieMOM.add(day, newValue);
		}

	}

}
